/*
 * Copyright (c) 1982, 1986, 1993
 *	The Regents of the University of California.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *	This product includes software developed by the University of
 *	California, Berkeley and its contributors.
 * 4. Neither the name of the University nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 *	@(#)in_proto.c	8.1 (Berkeley) 6/10/93
 */

#include "opt_inet.h"

#include <sys/param.h>
#include <sys/kernel.h>
#include <sys/socket.h>
#include <sys/domain.h>
#include <sys/protosw.h>
#include <sys/sysctl.h>

#include <net/if.h>
#include <net/route.h>

#include <netinet/in.h>
#include <netinet/in_systm.h>
#include <netinet/in6_var.h>
#include <netinet/ip.h>
#include <netinet/ip6.h>
#include <netinet/ipsec.h>
#include <netinet/in_pcb.h>
#include <netinet/ip_var.h>
#include <netinet/ip6_var.h>
#include <netinet/ip_icmp.h>
#include <netinet/icmp6.h>
#include <netinet/igmp_var.h>
#include <netinet/ip6_opts.h>
#include <netinet/ip6_policy.h>
#include <netinet/ip6_mob.h>
#include <netinet/tcp.h>
#include <netinet/tcp_timer.h>
#include <netinet/tcp_var.h>
#include <netinet/udp.h>
#include <netinet/udp_var.h>
/*
 * TCP/IP protocol family: IP, ICMP, UDP, TCP.
 */

#include "bti.h"
#include "dti.h"
#if NBTI > 0 || NDTI > 0
#include <netinet/if_bti6.h>
#endif

#include "tug.h"

extern	struct domain inet6domain;
static	struct pr_usrreqs nousrreqs;

#define INET6SW_NONE \
  0,		&inet6domain,	0,		0, \
  0,		0,		0,		0, \
  0,						   \
  0,		0,		0,		0, \
  &nousrreqs

struct protosw inet6sw[] = {
{ 0,		&inet6domain,	0,		0,
  0,		0,		0,		0,
  0,
  ip6_init,	0,		ip6flow_slowtimo, 0,
  &nousrreqs
},
{ SOCK_DGRAM,	&inet6domain,	IPPROTO_UDP,	PR_ATOMIC|PR_ADDR,
  udp_input,	0,		udp6_ctlinput,	ip6_ctloutput,
  0,
  0,		0,		0,		0,
  &udp6_usrreqs
},
{ SOCK_STREAM,	&inet6domain,	IPPROTO_TCP,
	PR_CONNREQUIRED|PR_IMPLOPCL|PR_WANTRCVD,
  tcp_input,	0,		tcp6_ctlinput,	tcp_ctloutput,
  0,
  0,		0,		0,		0,
  &tcp6_usrreqs
},
{ SOCK_RAW,	&inet6domain,	IPPROTO_RAW,	PR_ATOMIC|PR_ADDR,
  rip6_input,	0,		0,		rip6_ctloutput,
  0,
  0,		0,		0,		0,
  &rip6_usrreqs
},
{ SOCK_CONTROL,	&inet6domain,	IPPROTO_ICMPV6,	PR_ATOMIC|PR_ADDR,
  0,		0,		0,		rip6_ctloutput,
  0,
  0,		0,		0,		0,
  &rip6_usrreqs
},
{ SOCK_RAW,	&inet6domain,	IPPROTO_ICMPV6,	PR_ATOMIC|PR_ADDR,
  icmp6_input,	0,		rip6_ctlinput,	rip6_ctloutput,
  0,
  mld6_init,	mld6_fasttimo,	0,		0,
  &rip6_usrreqs
},
{ SOCK_RAW,	&inet6domain,	IPPROTO_RSVP,	PR_ATOMIC|PR_ADDR,
  rip6_input,	0,		0,		rip6_ctloutput,
  0,
  0,		0,		0,		0,
  &rip6_usrreqs
},
#if NBTI > 0 || NDTI > 0
{ SOCK_RAW,	&inet6domain,	IPPROTO_IPIP,	PR_ATOMIC|PR_ADDR,
  bti_input,	0,		0,		rip6_ctloutput,
  0,
  0,		0,		0,		0,
  &rip6_usrreqs
},
#endif
#if NTUG > 0
{ SOCK_RAW,	&inet6domain,	IPPROTO_IPV6,	PR_ATOMIC|PR_ADDR,
  ip6ip6_input,	0,		tug_ctlinput,	rip6_ctloutput,
  0,
  0,		0,		0,		0,
  &rip6_usrreqs
},
#endif
	/* place holder (must be before IP6_NHDR_HOP entry) */
{ INET6SW_NONE },
{ INET6SW_NONE },
{ INET6SW_NONE },
{ INET6SW_NONE },
{ INET6SW_NONE },
	/* next headers */
{ 0,		&inet6domain,	IP6_NHDR_FRAG,	0,
  frg6_input,	0,		opt6_ctlinput,	0,
  0,
  0,		0,		frg6_slowtimo,	frg6_drain,
  &nousrreqs
},
{ 0,		&inet6domain,	IP6_NHDR_HOP,	0,
  hop6_input,	0,		opt6_ctlinput,	0,
  0,
  opt6_init,	0,		0,		0,
  &nousrreqs
},
{ 0,		&inet6domain,	IP6_NHDR_RT,	0,
  rt6_input,	0,		opt6_ctlinput,	0,
  0,
  0,		0,		0,		0,
  &nousrreqs
},
	/* hack for mobile destination options */
{ SOCK_RAW,	&inet6domain,	IP6_NHDR_DOPT,	PR_ATOMIC|PR_ADDR,
  dopt6_input,	0,		opt6_ctlinput,	rip6_ctloutput,
  0,
  ipmob_init,	0,		0,		0,
  &rip6_usrreqs
},
{ 0,		&inet6domain,	IP6_NHDR_AUTH,	0,
  ah6_input,	0,		opt6_ctlinput,	0,
  0,
  0,		0,		0,		0,
  &nousrreqs
},
{ 0,		&inet6domain,	IP6_NHDR_ESP,	0,
  esp6_input,	0,		opt6_ctlinput,	0,
  0,
  0,		0,		0,		0,
  &nousrreqs
},
{ 0,		&inet6domain,	IP6_NHDR_IPCP,	0,
  ipcp6_input,	0,		opt6_ctlinput,	0,
  0,
  0,		0,		0,		0,
  &nousrreqs
},
{ SOCK_RAW,	&inet6domain,	IP6_NHDR_NONH,	PR_ATOMIC|PR_ADDR,
  end6_input,	0,		rip6_ctlinput,	rip6_ctloutput,
  0,
  0,		0,		0,		0,
  &rip6_usrreqs
},
	/* raw wildcard */
{ SOCK_RAW,	&inet6domain,	0,		PR_ATOMIC|PR_ADDR,
  rip6_input,	0,		0,		rip6_ctloutput,
  0,
  0,		0,		0,		0,
  &rip6_usrreqs
},
};

extern int in6_inithead __P((void **, int));

struct domain inet6domain =
    { AF_INET6, "internet6", 0, 0, 0, 
      inet6sw, &inet6sw[sizeof(inet6sw)/sizeof(inet6sw[0])], 0,
      in6_inithead, 64, sizeof(struct sockaddr_in6) * 2
    };

DOMAIN_SET(inet6);

SYSCTL_NODE(_net,      PF_INET6,	inet6,	CTLFLAG_RW, 0,
	"IPv6 Family");

SYSCTL_NODE(_net_inet6, IPPROTO_IP,	ipv6,	CTLFLAG_RW, 0,	"IPv6");
SYSCTL_NODE(_net_inet6, IPPROTO_ICMPV6,	icmpv6,	CTLFLAG_RW, 0,	"ICMPv6");
SYSCTL_NODE(_net_inet6, IPPROTO_RSVP,	rsvp,	CTLFLAG_RW, 0,	"RSVPv6");
SYSCTL_NODE(_net_inet6, IPPROTO_AH,	ipsec,	CTLFLAG_RW, 0,	"IPSec");

struct opt6sw opt6sw[] = {
	/* generic entries */
{ OPT6_A_SKIP,		OPT6_TYPE_GEN,
  opt6_skip_input,	opt6_gen_output, },
{ OPT6_A_DISC,		OPT6_TYPE_GEN,
  opt6_disc_input,	opt6_gen_output, },
{ OPT6_A_FERR,		OPT6_TYPE_GEN,
  opt6_ferr_input,	opt6_gen_output, },
{ OPT6_A_OERR,		OPT6_TYPE_GEN,
  opt6_oerr_input,	opt6_gen_output, },
	/* specific entries */
{ OPT6_PAD_1,		OPT6_TYPE_DEF,
  opt6_pad1_input,	opt6_pad1_output, },
{ OPT6_PAD_N,		OPT6_TYPE_DEF,
  opt6_padn_input,	opt6_gen_output, },
{ OPT6_ROUTER_ALERT, 	OPT6_TYPE_DEF,
    opt6_rtalert_input,	opt6_gen_output, },
{ OPT6_BINDING_UPD,	OPT6_TYPE_DEF,
    opt6_bndupd_input,	opt6_gen_output, },
{ OPT6_BINDING_ACK,	OPT6_TYPE_DEF,
    opt6_bndack_input,	opt6_gen_output, },
{ OPT6_BINDING_REQ,	OPT6_TYPE_DEF,
    opt6_bndreq_input,	opt6_gen_output, },
{ OPT6_HOME_ADDR,	OPT6_TYPE_DEF,
  opt6_haddr_input,	opt6_haddr_output, },
	/* paging entries 
	   BL mai 2000
	   I need two new option type : paging update option, 
	   and route update option
	*/
{ OPT6_ROUTE_UPD,			OPT6_TYPE_DEF,
    opt6_route_input,	opt6_gen_output, },
{ OPT6_PAGING_UPD,			OPT6_TYPE_DEF,
    opt6_paging_input,	opt6_gen_output, },
	/* spare entries */
{ 0,			OPT6_TYPE_NONE,
    opt6_skip_input,	opt6_gen_output, },
{ 0,			OPT6_TYPE_NONE,
    opt6_skip_input,	opt6_gen_output, },
{ 0,			OPT6_TYPE_NONE,
    opt6_skip_input,	opt6_gen_output, },
{ 0,			OPT6_TYPE_NONE,
    opt6_skip_input,	opt6_gen_output, },
{ 0,			OPT6_TYPE_NONE,
    opt6_skip_input,	opt6_gen_output, },
{ 0,			OPT6_TYPE_NONE,
    opt6_skip_input,	opt6_gen_output, },
{ 0,			OPT6_TYPE_NONE,
    opt6_skip_input,	opt6_gen_output, },
{ 0,			OPT6_TYPE_NONE,
    opt6_skip_input,	opt6_gen_output, },
	/* end */
{ 0,			OPT6_TYPE_END, },
};
