/*
 * Copyright (c) 1982, 1986, 1988, 1993
 *	The Regents of the University of California.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *	This product includes software developed by the University of
 *	California, Berkeley and its contributors.
 * 4. Neither the name of the University nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 *	@(#)ip_input.c	8.2 (Berkeley) 1/4/94
 */

#include "opt_inet.h"
#include "opt_ip6fw.h"

#include <sys/param.h>
#include <sys/systm.h>
#include <sys/mbuf.h>
#include <sys/domain.h>
#include <sys/protosw.h>
#include <sys/socket.h>
#include <sys/time.h>
#include <sys/kernel.h>
#include <sys/syslog.h>
#include <sys/sysctl.h>

#include <net/if.h>
#include <net/if_dl.h>
#include <net/route.h>

#include <netinet/in.h>
#include <netinet/in_systm.h>
#include <netinet/ip6.h>
#include <netinet/ip6_var.h>
#include <netinet/icmp6.h>
#include <netinet/ip6_opts.h>

#include <sys/socketvar.h>

u_char	opt6_extx[OPT6_MAX];
extern	struct opt6sw opt6sw[];

/*
 * IPv6 option initialization.
 */
void
opt6_init()
{
	struct opt6sw *op;
	int i;

	/* fill opt6_extx standard entries */

	op = &opt6sw[0];
	if ((op->op_ext != OPT6_A_SKIP) || (op->op_type != OPT6_TYPE_GEN))
		panic("opt6_init: skip");
	for (i = OPT6_A_SKIP; i < OPT6_A_DISC; i++)
		opt6_extx[i] = op - opt6sw;
	op++;
	if ((op->op_ext != OPT6_A_DISC) || (op->op_type != OPT6_TYPE_GEN))
		panic("opt6_init: disc");
	for (i = OPT6_A_DISC; i < OPT6_A_FERR; i++)
		opt6_extx[i] = op - opt6sw;
	op++;
	if ((op->op_ext != OPT6_A_FERR) || (op->op_type != OPT6_TYPE_GEN))
		panic("opt6_init: ferr");
	for (i = OPT6_A_FERR; i < OPT6_A_OERR; i++)
		opt6_extx[i] = op - opt6sw;
	op++;
	if ((op->op_ext != OPT6_A_OERR) || (op->op_type != OPT6_TYPE_GEN))
		panic("opt6_init: oerr");
	for (i = OPT6_A_OERR; i < OPT6_MAX; i++)
		opt6_extx[i] = op - opt6sw;
	op++;

	/* fill opt6_extx specific/defined entries */

	do {
		if (op->op_type == OPT6_TYPE_DEF)
			opt6_extx[op->op_ext] = op - opt6sw;
		op++;
	} while (op->op_type != OPT6_TYPE_END);
}

/*
 * Header option processing on incoming packets.
 */
int
hd6_inoptions(m, opts, alert)
	struct mbuf *m, *opts;
	int *alert;
{
	struct ipv6_h2hhdr *hp;
	struct opt6_any *op;
	int len, olen;
	int errpptr = sizeof(struct ip6_hdr);

	hp = mtod(opts, struct ipv6_h2hhdr *);
	op = (struct opt6_any *)&hp->ih6_pad1;
	len = ((hp->ih6_hlen + 1) * sizeof(*hp)) - (2 * sizeof(u_int8_t));

	while (len > 0) {
		olen = (*opt6sw[opt6_extx[op->o6any_ext]].op_input)
				(op, m, opts, alert);
		if (olen < 0)
			return (1);
		if (olen > len) {
			errpptr += (caddr_t)&op->o6any_len - (caddr_t)hp;
			m = ip6_insertoption(m, opts, NULL, IP6_INSOPT_RAW);
			icmp6_error(m, ICMP6_PARAM_PROB,
				    ICMP6_PARAMPROB_OPTION, errpptr);
			m_freem(opts);
			return (1);
		}
		op = (struct opt6_any *)((caddr_t)op + olen);
		len -= olen;
	}
	return (0);
}

int
opt6_pad1_input(op, m, opts, alert)
	struct opt6_any *op;
	struct mbuf *m, *opts;
	int *alert;
{
	return (1);
}

int
opt6_padn_input(op, m, opts, alert)
	struct opt6_any *op;
	struct mbuf *m, *opts;
	int *alert;
{
	return (op->o6any_len + sizeof(*op));
}
/* BL : mai 2000
   I need two new option to support paging
   route update and paging update function
   
   todo ; check the option 
 */
int
opt6_route_input(op, m, opts, alert)
	struct opt6_any *op;
	struct mbuf *m, *opts;
	int *alert;
{
	*alert |= DOPT6_HANEEDED;
	return (op->o6any_len + sizeof(*op));
}
int
opt6_paging_input(op, m, opts, alert)
	struct opt6_any *op;
	struct mbuf *m, *opts;
	int *alert;
{
	*alert |= DOPT6_HANEEDED;
	return (op->o6any_len + sizeof(*op));
}

int
opt6_rtalert_input(op, m, opts, alert)
	struct opt6_any *op;
	struct mbuf *m, *opts;
	int *alert;
{
	int errpptr;

	errpptr = (caddr_t)op - mtod(opts, caddr_t) + sizeof(struct ip6_hdr);
#ifdef DIAGNOSTIC
	if (alert == NULL)
		panic("opt6_rtalert_input");
#endif
	if ((opts->m_pkthdr.pktype != IP6_NHDR_HOP) || *alert)
		return (op->o6any_len + sizeof(*op));
	if (op->o6any_len != sizeof(u_int16_t))
		return (opts->m_len + 1);
	if (errpptr & 1) {
		/* alignment requirement is 2n */
		m = ip6_insertoption(m, opts, NULL, IP6_INSOPT_RAW);
		icmp6_error(m, ICMP6_PARAM_PROB,
			    ICMP6_PARAMPROB_HEADER, errpptr);
		m_freem(opts);
		return (-1);
	}
	*alert = (int)ntohs(*(u_int16_t *)(op + 1)) + 1;
	return (op->o6any_len + sizeof(*op));
}

int
opt6_bndupd_input(op, m, opts, alert)
	struct opt6_any *op;
	struct mbuf *m, *opts;
	int *alert;
{
	int errpptr;

	errpptr = (caddr_t)op - mtod(opts, caddr_t) + sizeof(struct ip6_hdr);
#ifdef DIAGNOSTIC
	if (alert == NULL)
		panic("opt6_bndupd_input");
#endif
	if (opts->m_pkthdr.pktype != IP6_NHDR_DOPT)
		return (opt6_oerr_input(op, m, opts, alert));
	/* check duplicates */
	if (*alert & DOPT6_HANEEDED)
		return (opt6_oerr_input(op, m, opts, alert));
	if (op->o6any_len < 8)
		return (opts->m_len + 1);
	/* alignment requirement is 4n+2 */
	if ((errpptr + 2 ) & 3) {
	    badalign:
		m = ip6_insertoption(m, opts, NULL, IP6_INSOPT_RAW);
		icmp6_error(m, ICMP6_PARAM_PROB,
			    ICMP6_PARAMPROB_HEADER, errpptr);
		m_freem(opts);
		return (-1);
	}
	/* check sub-options */
	if (op->o6any_len > 8) {
		int solen, len = op->o6any_len - 8;
		struct mob6_sopt_any *sop;

		errpptr += 2 + 8;
		sop = (struct mob6_sopt_any *)((caddr_t)(op + 1) + 8);
		do switch (sop->mso6_type) {
		    case MOB6_SUBOPT_PAD1:
			solen = 1;
			goto next;
		    case MOB6_SUBOPT_PADN:
		    default:
			solen = sizeof(*sop) + sop->mso6_len;
			goto next;
		    case MOB6_SUBOPT_UID:
			solen = sizeof(*sop) + sop->mso6_len;
			/* alignement requirement is 2n */
			if (errpptr & 1)
				goto badalign;
			goto next;
		    case MOB6_SUBOPT_COA:
			solen = sizeof(*sop) + sop->mso6_len;
			/* alignement requirement is 8n+6 */
			if ((errpptr + 2) & 3 /*7*/)
				goto badalign;
		    next:
			sop = (struct mob6_sopt_any *)((caddr_t)sop + solen);
			errpptr += solen;
			len -= solen;
		} while (len > 0);
	}
	*alert |= DOPT6_HANEEDED | DOPT6_AHNEEDED;
	return (op->o6any_len + sizeof(*op));
}

int
opt6_bndack_input(op, m, opts, alert)
	struct opt6_any *op;
	struct mbuf *m, *opts;
	int *alert;
{
	int errpptr;

	errpptr = (caddr_t)op - mtod(opts, caddr_t) + sizeof(struct ip6_hdr);
#ifdef DIAGNOSTIC
	if (alert == NULL)
		panic("opt6_bndack_input");
#endif
	if (opts->m_pkthdr.pktype != IP6_NHDR_DOPT)
		goto zap;
	if (op->o6any_len < 11)
		goto zap;
	if ((errpptr + 1) & 3) {
		/* alignment requirement is 4n+3 */
		goto zap;
	}
	*alert |= DOPT6_OTHER | DOPT6_AHNEEDED;
	/* check sub-options ? */
	return (op->o6any_len + sizeof(*op));

    zap:
	op->o6any_ext = OPT6_PAD_N;
	return (op->o6any_len + sizeof(*op));
}

int
opt6_bndreq_input(op, m, opts, alert)
	struct opt6_any *op;
	struct mbuf *m, *opts;
	int *alert;
{
#ifdef DIAGNOSTIC
	if (alert == NULL)
		panic("opt6_bndreq_input");
#endif
	if (opts->m_pkthdr.pktype != IP6_NHDR_DOPT)
		return (op->o6any_len + sizeof(*op));
	*alert |= DOPT6_OTHER;
	/* check sub-options */
	if (op->o6any_len > 0) {
		int solen, len = op->o6any_len;
		struct mob6_sopt_any *sop;
		int errpptr;

		sop = (struct mob6_sopt_any *)(op + 1);
		errpptr = (caddr_t)sop
			- mtod(opts, caddr_t)
			+ sizeof(struct ip6_hdr);

		do switch (sop->mso6_type) {
		    case MOB6_SUBOPT_PAD1:
			solen = 1;
			goto next;
		    case MOB6_SUBOPT_PADN:
		    default:
			solen = sizeof(*sop) + sop->mso6_len;
			goto next;
		    case MOB6_SUBOPT_UID:
			solen = sizeof(*sop) + sop->mso6_len;
			/* alignement requirement is 2n */
			if (errpptr & 1) {
				m = ip6_insertoption(m, opts,
						     NULL, IP6_INSOPT_RAW);
				icmp6_error(m, ICMP6_PARAM_PROB,
					    ICMP6_PARAMPROB_HEADER, errpptr);
				m_freem(opts);
				return (-1);
			}
		    next:
			sop = (struct mob6_sopt_any *)((caddr_t)sop + solen);
			errpptr += solen;
			len -= solen;
		} while (len > 0);
	}
	return (op->o6any_len + sizeof(*op));
}

int
opt6_haddr_input(op, m, opts, alert)
	struct opt6_any *op;
	struct mbuf *m, *opts;
	int *alert;
{
	struct ip6_hdr *ip = mtod(m, struct ip6_hdr *);
	struct in6_addr t, *ap;
	int errpptr;

	errpptr = (caddr_t)op - mtod(opts, caddr_t) + sizeof(struct ip6_hdr);
#ifdef DIAGNOSTIC
	if (alert == NULL)
		panic("opt6_haddr_input");
#endif
	if (opts->m_pkthdr.pktype != IP6_NHDR_DOPT)
		return (opt6_oerr_input(op, m, opts, alert));
	/* check duplicates */
	if (*alert & DOPT6_HASEEN)
		return (opt6_oerr_input(op, m, opts, alert));
	if (op->o6any_len < sizeof(struct in6_addr))
		return (opts->m_len + 1);
	if ((errpptr + 2 ) & 3 /*7*/) {
		/* alignment requirement is 8n+6 */
		m = ip6_insertoption(m, opts, NULL, IP6_INSOPT_RAW);
		icmp6_error(m, ICMP6_PARAM_PROB,
			    ICMP6_PARAMPROB_HEADER, errpptr);
		m_freem(opts);
		return (-1);
	}
	ap = (struct in6_addr *)(op + 1);
	COPY_ADDR6(ip->ip6_src, t);
	COPY_ADDR6(*ap, ip->ip6_src);
	COPY_ADDR6(t, *ap);
	*alert |= DOPT6_HASEEN;
	/* check sub-options ? */
	return (op->o6any_len + sizeof(*op));
}

int
opt6_skip_input(op, m, opts, alert)
	struct opt6_any *op;
	struct mbuf *m, *opts;
	int *alert;
{
	return (op->o6any_len + sizeof(*op));
}

int
opt6_disc_input(op, m, opts, alert)
	struct opt6_any *op;
	struct mbuf *m, *opts;
	int *alert;
{
	m_freem(opts);
	m_freem(m);
	return (-1);
}

int
opt6_ferr_input(op, m, opts, alert)
	struct opt6_any *op;
	struct mbuf *m, *opts;
	int *alert;
{
	int errpptr;

	errpptr = (caddr_t)op - mtod(opts, caddr_t) + sizeof(struct ip6_hdr);
	m = ip6_insertoption(m, opts, NULL, IP6_INSOPT_RAW);
	icmp6_error(m, ICMP6_PARAM_PROB, ICMP6_PARAMPROB_OPTION, errpptr);
	m_freem(opts);
	return (-1);
}

int
opt6_oerr_input(op, m, opts, alert)
	struct opt6_any *op;
	struct mbuf *m, *opts;
	int *alert;
{
	struct ip6_hdr *ip = mtod(m, struct ip6_hdr *);

	if (m->m_flags & (M_BCAST|M_MCAST) ||
	    IS_MULTIADDR6(ip->ip6_dst)) {
		m_freem(opts);
		m_freem(m);
		return (-1);
	}
	return (opt6_ferr_input(op, m, opts, alert));
}

/*
 * Header option processing on outgoing packets.
 */
int
hd6_outoptions(m, cp)
	struct mbuf *m;
	void *cp;
{
	struct ipv6_h2hhdr *hp;
	struct opt6_any *op;
	int len, olen;

	hp = (struct ipv6_h2hhdr *)cp;
	op = (struct opt6_any *)&hp->ih6_pad1;
	len = (hp->ih6_hlen + 1) * sizeof(*hp) - (2 * sizeof(u_int8_t));

	while (len > 0) {
		olen = (*opt6sw[opt6_extx[op->o6any_ext]].op_output) (op, m);
		if (olen < 0)
			return (1);
		if (olen > len)
			return (1);
		op = (struct opt6_any *)((caddr_t)op + olen);
		len -= olen;
	}
	return (0);
}

int
opt6_pad1_output(op, m)
	struct opt6_any *op;
	struct mbuf *m;
{
	return (1);
}

int
opt6_haddr_output(op, m)
	struct opt6_any *op;
	struct mbuf *m;
{
	struct ip6_hdr *ip = mtod(m, struct ip6_hdr *);
	struct in6_addr t, *ap;

	if (ip->ip6_nxt != IP6_NHDR_DOPT)
		return (-1);
	if (op->o6any_len < sizeof(struct in6_addr))
		return (m->m_len + 1);
	ap = (struct in6_addr *)(op + 1);
	COPY_ADDR6(ip->ip6_src, t);
	COPY_ADDR6(*ap, ip->ip6_src);
	COPY_ADDR6(t, *ap);
	return (op->o6any_len + sizeof(*op));
}

int
opt6_gen_output(op, m)
	struct opt6_any *op;
	struct mbuf *m;
{
	return (op->o6any_len + sizeof(*op));
}
