/*
 * Copyright (c) 1982, 1986, 1993
 *	The Regents of the University of California.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *	This product includes software developed by the University of
 *	California, Berkeley and its contributors.
 * 4. Neither the name of the University nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 *	@(#)ip_icmp.h	8.1 (Berkeley) 6/10/93
 */

#ifndef _NETINET_IP6_OPTS_H_
#define _NETINET_IP6_OPTS_H_

/*
 * Definitions for IPv6 options.
 */
#define OPT6_PAD_1		0	/* Single Pad */
#define OPT6_PAD_0		OPT6_PAD_1
#define OPT6_PAD_N		1	/* Multiple Pad */
#define OPT6_JUMBO		194	/* Jumbo-Payload */
#define OPT6_NSAPA		195	/* NSAP Addresses */
#define OPT6_TUN_ENC_LIM	4	/* Tunnel Encapsulation Limit */
#define OPT6_ROUTER_ALERT	5	/* Router Alert */
#define OPT6_BINDING_UPD	198	/* Binding Update */
#define OPT6_BINDING_ACK	7	/* Binding Acknowledge */
#define OPT6_BINDING_REQ	8	/* Binding Request */
#define OPT6_HOME_ADDR		201	/* Home Address */
#define OPT6_NIMROD_EPID	138	/* Nimrod Endpoint Identification */
#define OPT6_MAX		256	/* Max */
/* BL : august 2000
   I had two new option to support paging
 */
#define OPT6_ROUTE_UPD		9
#define OPT6_PAGING_UPD		10

/*
 * Macros on type bits.
 */
#define	OPT6_ACTION(t)		((t)&0xc0)	/* action */
#define OPT6_A_SKIP		0x00		/* skip over */
#define OPT6_A_DISC		0x40		/* discard */
#define OPT6_A_FERR		0x80		/* already send error */
#define OPT6_A_OERR		0xc0		/* send error */
#define OPT6_RTCHANGE(t)	((t)&0x20)	/* change en-route */

/*
 * Options and Extensions layouts.
 */

struct opt6_any {			/* common header */
	u_int8_t	o6any_ext;	/* extension type */
	u_int8_t	o6any_len;	/* length */
};

struct opt6_jbo {			/* Jumbo-Payload */
	u_int8_t	jbo_ext;	/* extension type (194) */
	u_int8_t	jbo_len;	/* length (4) */
	u_int8_t	jbo_plen[4];	/* payload length */
};

struct opt6_nsapa {			/* NSAP Addresses */
	u_int8_t	nsapa_ext;	/* extension type (195) */
	u_int8_t	nsapa_len;	/* length (2..42) */
	u_int8_t	nsapa_slen;	/* source NSAPA length (0..20) */
	u_int8_t	nsapa_dlen;	/* destination NSAPA length (0..20) */
	u_int8_t	nsapa_src[20];	/* source NSAPA */
	u_int8_t	nsapa_dst[20];	/* destination NSAPA */
}; 

struct opt6_tel {			/* Tunnel Encapsulation Limit */
	u_int8_t	tel_ext;	/* extension type (4) */
	u_int8_t	tel_len;	/* length (1) */
	u_int8_t	tel_tel;	/* value */
};

struct opt6_ra {			/* Router-Alert */
	u_int8_t	ra_ext;		/* extension type (5) */
	u_int8_t	ra_len;		/* length (2) */
	u_int8_t	ra_code[2];	/* code */
};
#define OPT6_RA_GROUP	0		/* ICMPv6 Group Membership */
#define OPT6_RA_RSVP	1		/* RSVP */
#define OPT6_RA_ACTNET	2		/* Active Networks */

struct opt6_bnd_upd {			/* Binding Update */
	u_int8_t	bndu_ext;	/* extension type (198) */
	u_int8_t	bndu_len;	/* length (8+) */
	u_int8_t	bndu_flg;	/* flags */
	u_int8_t	bndu_plen;	/* prefix length */
	u_int8_t	bndu_seq[2];	/* sequence number */
	u_int8_t	bndu_life[4];	/* lifetime */
};
#define OPT6_BNDU_ACK	0x80		/* Request an acknowledgement */
#define OPT6_BNDU_HR	0x40		/* Home registration */
#ifdef nomore
#define OPT6_BNDU_COA	0x20		/* Care-of address is present */
#endif
#define OPT6_BNDU_RT	0x20		/* Mobile is a router */
/* new */
#define OPT6_BNDU_DAD	0x10		/* Mobile wants a proxy DAD */

struct opt6_bnd_ack {			/* Binding Acknowledge */
	u_int8_t	bnda_ext;	/* extension type (7) */
	u_int8_t	bnda_len;	/* length (11+) */
	u_int8_t	bnda_status;	/* status */
	u_int8_t	bnda_seq[2];	/* sequence number */
	u_int8_t	bnda_life[4];	/* lifetime */
	u_int8_t	bnda_refresh[4];/* refresh */
};
#define OPT6_BNDA_ACCEPT	0	/* Accepted */
#define OPT6_BNDA_UNSPEC	128	/* Reason unspecified */
#define OPT6_BNDA_ADMIN		130	/* Administratively prohibited */
#define OPT6_BNDA_RES		131	/* Insufficient resources */
#define OPT6_BNDA_UNSUP		132	/* Home registration not supported */
#define OPT6_BNDA_SUBNET	133	/* Not home subnet */
/* old */
#define OPT6_BNDA_DYNDIS	135	/* Dynamic HA address discovery */
#define OPT6_BNDA_BADIFF	136	/* Incorrect interface identifier */
#define OPT6_BNDA_REFUSED	137	/* Not home agent for this host */
/* new */
#define OPT6_BNDA_DADFAIL	138	/* Duplicate Address Detected */

struct opt6_bnd_req {			/* Binding Request */
	u_int8_t	bndq_ext;	/* extension type (8) */
	u_int8_t	bndq_len;	/* length (0+) */
};

struct opt6_homadd {			/* Home Address */
	u_int8_t	ha_ext;		/* extension type (201) */
	u_int8_t	ha_len;		/* length (16+) */
	u_int8_t	ha_addr[16];	/* home address */
};

/* Mobility sub-options */

#define MOB6_SUBOPT_PAD1	0	/* Pad1 */
#define MOB6_SUBOPT_PADN	1	/* PadN */
#define MOB6_SUBOPT_UID		2	/* Unique Identifier */
/* 3 was for Home Agents List Sub-Option */
#define MOB6_SUBOPT_COA		4	/* Alternate Care-of Address */ 

struct mob6_sopt_any {			/* common header */
	u_int8_t	mso6_type;	/* type */
	u_int8_t	mso6_len;	/* length */
};

struct mob6_sopt_uid {			/* Unique identifier */
	u_int8_t	msuid_type;	/* type (2) */
	u_int8_t	msuid_len;	/* length (2) */
	u_int8_t	msuid_uid[2];	/* unique identifier */
};

struct mob6_sopt_coa {			/* Alternate care-of address */
	u_int8_t	mscoa_type;	/* type (4) */
	u_int8_t	mscoa_len;	/* length (16) */
	u_int8_t	mscoa_coa[16];	/* alternate COA */
};

/* Alert values for incoming mobility destination options */

#define DOPT6_NONE	0		/* Nothing interesting */
#define DOPT6_HASEEN	1		/* Home Address seen */
#define DOPT6_HANEEDED	2		/* Home Address needed */
#define DOPT6_HAUPD	3		/* Home Address & Binding Update */
#define DOPT6_OTHER	4		/* Other cases (or'ed) */
#define DOPT6_AHNEEDED	8		/* IPSec AH needed */
#define DOPT6_MMSK	15		/* Mobility Mask */
#define DOPT6_MASK	15		/* Mask */
#define DOPT6_MFSHT	8		/* Mbuf flag shift */

/*
 * Per option function array
 *  arguments are option, packet, headers and on input router alert pointer.
 */
struct opt6sw {
	u_int8_t op_ext;
	u_short op_type;
	int (*op_input) __P((struct opt6_any *, struct mbuf *,
			     struct mbuf *, int *));
	int (*op_output) __P((struct opt6_any *, struct mbuf *));
	/* TODO: add alignment constraints and a check function ?? */
};

/*
 * Definitions for Router Renumbering operations
 */

#define RTRN6_FLG_TEST		0x80	/* Test message flag */
#define RTRN6_FLG_RSLT		0x40	/* Result wanted flag */
#define RTRN6_FLG_ALL		0x20	/* All interfaces flag */
#define RTRN6_FLG_SITE		0x10	/* in Site only flag */
#define RTRN6_FLG_PROC		0x08	/* Processes flag */
#define RTRN6_FLG_ERR		0x04	/* Error flag */

#define RTRN6_PRFOP_ADD		1	/* Prefix Add operation */
#define RTRN6_PRFOP_CHANGE	2	/* Prefix Change operation */
#define RTRN6_PRFOP_SETGLB	3	/* Prefix Set-Global operation */

#define RTRN6_UFLG_V	0x80000000	/* valid lifetime in real time */
#define RTRN6_UFLG_P	0x40000000	/* preferred lifetime in real time */

struct rtrn6_match {			/* Match-Prefix */
	u_int8_t	match_code;	/* opcode (RTRN6_PRFOP_*) */
	u_int8_t	match_len;	/* length (4N+3) */
	u_int8_t	match_ord;	/* ordinal */
	u_int8_t	match_plen;	/* prefix match length (0..128) */
	u_int32_t	match_res;	/* reserved */
	struct in6_addr	match_pref;	/* prefix */
};

struct rtrn6_use {			/* Use-Prefix */
	u_int8_t	use_plen;	/* prefix use length (0..128) */
	u_int8_t	use_klen;	/* prefix keep length (0..128) */
	u_int8_t	use_flgmsk;	/* flag mask (for RA) */
	u_int8_t	use_ramsk;	/* flags (for RA) */
	u_int32_t	use_ilife;	/* (in)valid lifetime */
	u_int32_t	use_dlife;	/* (deprecation) preferred lifetime */
	u_int32_t	use_flags;	/* flags (RTRN6_UFLG_?) */
	struct in6_addr	use_pref;	/* prefix */
};

struct rtrn6_report {			/* Match-Report */
	u_int16_t	rep_res;	/* reserved */
	u_int8_t	rep_ord;	/* ordinal */
	u_int8_t	rep_plen;	/* matched prefix length */
	u_int32_t	rep_ifidx;	/* interface index (SNMP IfGroup) */
	struct in6_addr	rep_pref;	/* matched prefix */
};

#ifdef KERNEL

#define OPT6_TYPE_NONE	0
#define OPT6_TYPE_GEN	1
#define OPT6_TYPE_DEF	2
#define OPT6_TYPE_END	3

int	opt6_pad1_input __P((struct opt6_any *,
		struct mbuf *, struct mbuf *, int *));

int	opt6_padn_input __P((struct opt6_any *,
		struct mbuf *, struct mbuf *, int *));

/* BL : august 2000
   I add two new functions to support paging
 */

int	opt6_route_input __P((struct opt6_any *,
		struct mbuf *, struct mbuf *, int *));

int	opt6_paging_input __P((struct opt6_any *,
		struct mbuf *, struct mbuf *, int *));

int	opt6_rtalert_input __P((struct opt6_any *,
		struct mbuf *, struct mbuf *, int *));

int	opt6_bndupd_input __P((struct opt6_any *,
		struct mbuf *, struct mbuf *, int *));

int	opt6_bndack_input __P((struct opt6_any *,
		struct mbuf *, struct mbuf *, int *));

int	opt6_bndreq_input __P((struct opt6_any *,
		struct mbuf *, struct mbuf *, int *));

int	opt6_haddr_input __P((struct opt6_any *,
		struct mbuf *, struct mbuf *, int *));

int	opt6_skip_input __P((struct opt6_any *,
		struct mbuf *, struct mbuf *, int *));

int	opt6_disc_input __P((struct opt6_any *,
		struct mbuf *, struct mbuf *, int *));

int	opt6_ferr_input __P((struct opt6_any *,
		struct mbuf *, struct mbuf *, int *));

int	opt6_oerr_input __P((struct opt6_any *,
		struct mbuf *, struct mbuf *, int *));

int	opt6_pad1_output __P((struct opt6_any *, struct mbuf *));

int	opt6_haddr_output __P((struct opt6_any *, struct mbuf *));

int	opt6_gen_output __P((struct opt6_any *, struct mbuf *));
#endif
#endif
