/* Reception */

#include "ndpd.h"
#include "mobility.h"

#ifdef uMIP
#include "../ndpd-uhmma/uhmma.h"
#endif

#define MAXPACKET	8192	/* max packet size */
static char buf[MAXPACKET];

static struct in6_addr *get_altcoa(u_char *, int);
static int router_recv_destopt(struct ip6_hdr *, u_char *, struct in6_addr *);

/* Receive a destination option header */

void
recv_destopt()
{
	register struct ip6_hdr *ip;
	register struct ipv6_dopthdr *dop;
	register u_char *cp, *ep, *upd = 0;
	struct in6_addr *careof = NULL;
	struct in6_addr *altcoa = NULL;
	int len;
#ifdef uMIP
    int UHMMA_reg = 0;
#endif

	if ((len = read(m, buf, sizeof(buf))) < 0) {
		if (verbose)
			logperror("read (destopt)");
		return;
	}
	if (len == 0) {
		if (verbose)
			logerr("read destopt return 0");
		return;
	}

	ip = (struct ip6_hdr *)buf;
	if (len < sizeof(*ip) + sizeof(*dop)) {
		if (verbose)
			logerr("destopt: too short (%d)", len);
		return;
	}
	if (ip->ip6_nxt != IP6_NHDR_DOPT) {
		if (verbose)
			logerr("destopt: bad nh (%d)", ip->ip6_nxt);
		return;
	}

	len -= sizeof(struct ip6_hdr);
	dop = (struct ipv6_dopthdr *)(ip + 1);

	if (((int)(dop->io6_hlen) + 1) * 8 < len)
		len = ((int)(dop->io6_hlen) + 1) * 8;

	cp = ((u_char *)dop) + 2 * sizeof(u_int8_t);
	ep = ((u_char *)dop) + len;

	while (cp < ep)
		switch (*cp) {
		    case OPT6_PAD_1:
			cp++;
			break;

		    default:
		    case OPT6_PAD_N:
			cp++;
			len = *cp++;
			cp += len;
			break;

		    case OPT6_BINDING_UPD:
			cp++;
			len = *cp++;
			/* need home and careof addresses: defer! */
			upd = cp;
			if (len > 8)
				altcoa = get_altcoa(cp + 8, len - 8); 
			cp += len;
			break;

#ifdef uMIP
		    case OPT6_ROUTE_UPD:
			cp++;
			len = *cp++;
			/* need home and careof addresses: defer! */
			upd = cp;
			cp += len;
			UHMMA_reg = 1;
			break;		    

		    case OPT6_PAGING_UPD:
			cp++;
			len = *cp++;
			/* need home and careof addresses: defer! */
			upd = cp;
			cp += len;
			UHMMA_reg = 2;
			break;		    
#endif

		    case OPT6_BINDING_ACK:
			cp++;
			len = *cp++;
			recv_bnd_ack(ip, cp, len);
			cp += len;
			break;

		    case OPT6_BINDING_REQ:
			cp++;
			len = *cp++;
#if defined (uMIP)||(uMIPBS)||(UHMMA)
/* just add the option in the call of recv_bnd_req, to receive sub-options */
			recv_bnd_req(ip,cp,len);
#else
			recv_bnd_req(ip);
#endif
			cp += len;
			break;

		    case OPT6_HOME_ADDR:
			cp++;
			len = *cp++;
			/* addresses already swapped by kernel */
			careof = (struct in6_addr *)cp;
			cp += len;
			break;
		}

	if (cp != ep) {
		if (verbose)
			logerr("destop: garbage %d bytes", cp - ep);
		return;
	}
#ifdef uMIP
	/* -------------------------------------
	
	    uhmma registration

	    so check if this router is a mobility agent (mode 1 or 2). 
	    If it is not, send back an acknowledgement to the sender.

	----------------------------------------*/
        if(UHMMA_reg == 1)
        {
            if((MA_op == 1)||(MA_op == 2))
            {
                recv_uhmma_reg (ip, &upd, careof);
            }
            else
            {
		struct mobile mop;
		COPY_ADDR6(ip->ip6_dst,mop.me);
		COPY_ADDR6(ip->ip6_src,mop.careof);
    		send_bnd_ack(&mop, OPT6_BNDA_MA_REFUSED, 0);
            }
	    return;
        }
	if(UHMMA_reg == 2)
	{
            if((MA_op == 1)||(MA_op == 2))
            {
                recv_old_position (ip, upd, careof);
            }
            else
            {
		struct mobile mop;
		COPY_ADDR6(ip->ip6_dst,mop.me);
		COPY_ADDR6(ip->ip6_src,mop.careof);
    		send_bnd_ack(&mop, OPT6_BNDA_MA_REFUSED, 0);
            }
	    return;
	}
#endif	//uMIP
	if (upd) {
		if (careof == NULL) {
			if (verbose)
				logerr("destop: binding update without"
				       "home address?");
			return;
		}
		/* an alternate care-of is better */
		if (altcoa) {
			if (verbose && !IN6_ARE_ADDR_EQUAL(altcoa, careof))
				logtrace("real careof is %s", inet6pr(altcoa));
			careof = altcoa;
		}
		if (router_recv_destopt(ip, upd, careof))
			return;
#ifdef uMIP
    /* ------------------------------------
    
	the flags in the bu are FORWARD_FLAG|0x20 
	
	the BU is sent by a MA to ask to forward packet to a mobile witch
	was in this domain, and now is in an other domain
	we can receive forward request in mode 1 AND in mode 2
	because a mh can run mode1 in a mode2 domain.

    ---------------------------------------*/
		if(*upd == 0x22)/* FORWARD_FLAG|0x20 */
		{
		    if((MA_op==1)||(MA_op==2))
		    {
			recv_forward(ip, upd,careof);
		    }
		    else
		    {
			struct mobile mop;
			COPY_ADDR6(ip->ip6_dst,mop.me);
			COPY_ADDR6(*careof,mop.careof);
    			send_bnd_ack(&mop, OPT6_BNDA_MA_REFUSED, 0);
		    }
		    return;
		}
    /* ---------------------------------------
    
	the flags in the BU are SMOOTH_FLAG
	
	the BU is sent by a MA to ask this router to forward
	packet to the new mobile host's position AND to transmit
	the correspondents list
    
    ------------------------------------------ */
		if(*upd == 0x24)/* SMOOTH_FLAG|0x20 */
		{
		    if(MA_op==2)
		    {
			recv_smooth(ip, upd, careof);
		    }
		    else
		    {
			/* the destination is not a mobile 
			   but we use a mobile structure to
			   send the BACK packet */
			struct mobile mop;
			COPY_ADDR6(ip->ip6_dst,mop.me);
			COPY_ADDR6(*careof,mop.careof);
    			send_bnd_ack(&mop, OPT6_BNDA_MA_REFUSED, 0);
		    }
		    return;
		}
#endif
		if (*upd & OPT6_BNDU_HR)
			recv_home_reg(ip, upd, careof);
		else
			recv_bnd_upd(ip, upd, careof);
	}
}

static struct in6_addr *
get_altcoa(cp, len)
	u_char *cp;
	int len;
{
	struct mob6_sopt_coa *sop;
	int solen = 0;

	while (len > 0) {
		sop = (struct mob6_sopt_coa *)(cp + solen);
		switch (sop->mscoa_type) {
		    case MOB6_SUBOPT_PAD1:
			solen = 1;
			len -= solen;
			break;
		    case MOB6_SUBOPT_PADN:
		    default:
			solen = 2 + sop->mscoa_len;
			len -= solen;
			break;
		    case MOB6_SUBOPT_COA:
			if (sop->mscoa_len != sizeof(struct in6_addr)) {
				if (verbose)
					logerr("get_altcoa: "
					       "bad alternate care-of");
				return NULL;
			}
			if (len < 2 + sop->mscoa_len) {
				if (verbose)
					logerr("get_altcoa: garbage for "
					       "alternate care-of");
				return NULL;
			}
			return (struct in6_addr *)sop->mscoa_coa;
		}
		if (len < 0) {
			if (verbose)
				logerr("get_altcoa: get garbage");
			return NULL;
		}
	}
	return NULL;
}

#ifdef NDPD_ROUTER

/* Special handler for a router because of anycasts */

static int
router_recv_destopt(ip, upd, careof)
	struct ip6_hdr *ip;
	u_char *upd;
	struct in6_addr *careof;
{
	struct interface *ifp = NULL;
	struct prefix *pp = NULL;
	struct anycast *ap = NULL;

	if (IN6_IS_ADDR_MULTICAST(&ip->ip6_dst))
		goto found;

	FORALLIFP(ifp)
	    FORALLPP(pp, ifp)
		FORALLPANY(ap, pp)
		    if (IN6_ARE_ADDR_EQUAL(&ap->addr, &ip->ip6_dst))
			goto found;
	FORALLIFP(ifp)
	    FORALLOANY(ap, ifp)
		if (IN6_ARE_ADDR_EQUAL(&ap->addr, &ip->ip6_dst))
		    goto found;
	return 0;

	/* Not really for us! */

    found:

	/* Try home agent discovery */

#ifdef HOME_AGENT
	if ((*upd & OPT6_BNDU_HR) &&
	    (ifp != NULL) && (pp != NULL) && (ap != NULL))
	    recv_ha_disc(ip, upd, careof, ap);
#endif

	/* Just drop it ? */

	return 1;
}
#else

/* Void router_recv_destopt() routine */

static int
router_recv_destopt(ip, upd, careof)
	struct ip6_hdr *ip;
	u_char *upd;
	struct in6_addr *careof;
{
	/* Drop too stupid things! */

	if (IN6_IS_ADDR_MULTICAST(&ip->ip6_dst))
		return 1;
	return 0;
}
#endif
