/* Binding request processing */

#ifdef MOBILE

#include "ndpd.h"
#include "mobility.h"

#if defined (uMIP)||(uMIPBS)||(UHMMA)
#include "../ndpd-uhmma/uhmma.h"

/* Receive a binding request */
void recv_bnd_req(ip, cp, len)
struct ip6_hdr *ip;
u_char *cp;
int len;
#else
void
recv_bnd_req(ip)
	struct ip6_hdr *ip;
#endif
{
	register struct corr *cop;

	(void)gettimeofday(&now, NULL);

	if (verbose) {
		logtrace("receive binding request from %s",
			 inet6pr(&ip->ip6_src));
	}
#ifdef UHMMA
/* If the mobile host runs the uhmma code, 
binding updates are sent by the mobility agent, 
so all binding request received must be dropped.
*/
    if(global.mobility_mode == UHMMA_MODE2)
    {
        if (verbose)
                logtrace("binding request from %s is dropped",
                         inet6pr(&ip->ip6_src));
		return;
    }
#endif

	/* look for a correspondent entry */

	FORALLCOR(cop)
		if ((cop->type & CORR_USEBND) &&
		    (IN6_ARE_ADDR_EQUAL(&ip->ip6_src, &cop->addr)))
			break;

	if (cop == NULL) {
		/* got no correspondent entry */

		if (verbose)
			logtrace("no matching correspondent?");
		return;
	}

	/* should verify the routing header too */

	if (!IN6_ARE_ADDR_EQUAL(&ip->ip6_dst, &cop->home)) {
		if (verbose)
			logtrace("bad destination %s != %s",
				 inet6pr(&ip->ip6_dst), inet6pr(&cop->home));
	}

	/* cancel refresh */

	if (cop->tp) {
		kill_timeout(cop->tp);
		cop->tp = NULL;
	}

	/* strange case where we are trying to get an ack too */

	if (cop->rettp) {
		if (verbose)
			logtrace("%s sends a req in place of an ack?",
				 inet6pr(&ip->ip6_dst));
		cancel_retrans(cop);
	}

	/* send an update */

	cop->seq++;

	if (cop->flags & OPT6_BNDU_HR)
		send_home_reg(cop);
	else
		send_bnd_upd(cop);

	if (cop->flags & OPT6_BNDU_ACK)
		start_retrans(cop);

	return;
}
#endif
