/* Security Association management for a mobile node */

#include "ndpd.h"
#include "mobility.h"

#ifdef MOBILE

static void probe_ipsec(struct corr *);

void
mget_security(cop)
	struct corr *cop;
{
	if (is_secure == -1)
		is_secure = check_ipsec();
	if (!is_secure)
		return;

	if ((cop->type & CORR_USEBND) == 0)
		return;

	probe_ipsec(cop);
}

void
mdel_security(cop)
	struct corr *cop;
{
	if (cop->sap)
		free(cop->sap);
	cop->sap = NULL;
}

static void
probe_ipsec(cop)
	struct corr *cop;
{
	struct sockaddr_in6 sin;
	int d, cc;

	if (verbose)
		logtrace("probe IPsec SA %s -> %s",
			 inet6pr(&cop->home), inet6pr(&cop->addr));

	/* get a fresh socket */

	if ((d = socket(PF_INET6, SOCK_RAW, IPPROTO_DSTOPTS)) < 0) {
		logperror("socket (probe IPsec)");
		return;
	}
	(void) shutdown(d, 2);

	/* bind it to our home address */

	bzero((caddr_t)&sin, sizeof(sin));
	sin.sin6_family = AF_INET6;
	sin.sin6_len = sizeof(sin);
	COPY_ADDR6(cop->home, sin.sin6_addr);

	if (bind(d, (struct sockaddr *)&sin, sizeof(sin)) < 0) {
		logperror("bind (probe IPsec)");
		(void) close(d);
		return;
	}

	/* connect it to the correspondent */

	COPY_ADDR6(cop->addr, sin.sin6_addr);

	if (connect(d, (struct sockaddr *)&sin, sizeof(sin)) < 0) {
		logperror("connect (probe IPsec)");
		(void) close(d);
		return;
	}

	/* set the security policy */

	if (setsockopt(d, IPPROTO_IPV6, IPV6_IPSEC_POLICY,
		       ipsec_policy, ipsec_len) < 0) {
		logperror("setsockopt (probe IPsec / get)");
		(void) close(d);
		return;
	}

	/* probe the security policy */

	cc = setsockopt(d, IPPROTO_IPV6, IPV6_IPSEC_POLICY, NULL, 0);

	if (cc < 0) {
		if (cc != ENOENT)
			logperror("setsockopt (probe IPsec / probe)");
		else if (verbose)
			logtrace("probe IPsec: acquiring");
	}

	(void) close(d);
}
#endif
