/* Dummy code for a mobile node */

#ifndef MOBILE

#include "ndpd.h"
#include "mobility.h"

#ifdef uMIPBS
#include "../ndpd-uhmma/uhmma.h"
#endif
#ifdef uMIP
#include "../ndpd-uhmma/uhmma.h"
#endif

/* Receive a binding acknowledgement */

void
recv_bnd_ack(ip, cp, len)
	struct ip6_hdr *ip;
	u_char *cp;
	int len;
{
	u_char status;
	u_short seq;
	u_int life, refresh;
	char *msg;
#ifdef uMIP
    u_char *svg = cp;
#endif

	(void)gettimeofday(&now, NULL);

	GET8(status, cp);
	GET16(seq, cp);
	GET32(life, cp);
	GET32(refresh, cp);

	switch (status) {
	    case OPT6_BNDA_UNSPEC:
		msg = "Reason unspecified";
		break;

	    case OPT6_BNDA_ADMIN:
		msg = "Administratively prohibited";
		break;

	    case OPT6_BNDA_RES:
		msg = "Insufficient resources";
		break;

	    case OPT6_BNDA_UNSUP:
		msg = "Home registration not supported";
		break;

	    case OPT6_BNDA_SUBNET:
		msg = "Not home subnet";
		break;

	    case OPT6_BNDA_DYNDIS:
		msg = "Dynamic home agent address discovery response";
		break;

	    case OPT6_BNDA_BADIFF:
		msg = "Incorrect interface identifier length";
		break;

	    case OPT6_BNDA_REFUSED:
		msg = "Not home agent for this host";
		break;

#ifdef uMIP
	    case OPT6_BNDA_CN_LIST_KO:
	    case OPT6_BNDA_CN_LIST_OK:
		recv_cn_list(&ip->ip6_src,svg,len);
		return;
#endif

	    case OPT6_BNDA_DADFAIL:
		msg = "Duplicate Address Detected";
		break;

	    default:
		msg = "Unknown reason?";
	}

	if (verbose) {
		logtrace("receive binding ack from %s "
			 "seq=%d life=%u refresh=%u",
			 inet6pr(&ip->ip6_src),
			 (int)seq, life, refresh);
		if (status != OPT6_BNDA_ACCEPT)
			logtrace("binding ack failed (%d): %s", status, msg);
	}

	if (!mobile) {
		if (verbose)
			logerr("receive binding ack: not a mobile?!");
		return;
	}
}

/* Receive a binding request */

#if defined (uMIPBS)||(uMIP)
/* add two parameters */
void
recv_bnd_req(ip, cp, len)
	struct ip6_hdr *ip;
	u_char * cp;
	int len;
#else
void
recv_bnd_req(ip)
	struct ip6_hdr *ip;
#endif
{
	(void)gettimeofday(&now, NULL);

	if (verbose) {
		logtrace("receive binding request from %s",
			 inet6pr(&ip->ip6_src));
	}

	if (!mobile) {
		if (verbose)
			logerr("receive binding request: not a mobile?!");
#ifndef uMIPBS
		return;
#endif
	}
#ifdef uMIPBS
	/* we use the BReq to transport the MA annonce 
	    the MA annonce is for BS only, so just test if the receiver is
	    a BS and if the BReq contains a sub-option and procede.

	    DONT FORGET TO comment the return upper
	*/
    if ((cp !=NULL) && (*cp == SUB_OPT6_MA))
	recv_MA_data(cp);
#endif
}

#endif
