/* dump mobility data to screen when ndpd-deamon runs in debug mode */
/* share between UHMMA and uMIP */

#include "ndpd.h"
#include "../ndpd-mobility/mobility.h"
#include "../ndpd-uhmma/uhmma.h"

#ifdef uMIP
void dump_listMH();
void dump_listMA();
void dump_listBS();

/*
ndpd-router is running in debug mode, the user presses CTRL+C,
we have to dump all interesting data.
Here the router is a Mobility Agent.

called by : main
call : dump_listMH
*/
void dump_MA()
{
    logtrace("\tThis router is a Mobility Agent");
    logtrace("\tMobility Mode 	   : %d",MA_op);
    logtrace("\tVirtual Interface  : %s",VirtualNetwork.name);
    logtrace("\tRoot Interface     : %s",VirtualNetwork.root);
    logtrace("\tVirtual Prefix     : %s/%d",inet6pr(&VirtualNetwork.pref),
	VirtualNetwork.plen);
    logtrace("\tCharge             : %d",charge);
    logtrace("\t--------------------------------------> mobility agent's list");
    dump_listMA();
    logtrace("\t--------------------------------------> base station's list");
    dump_listBS();
    logtrace("\t--------------------------------------> mobile nodes' list");
    dump_listMH();
}
/* 
we print all the MH in the managed MH list

called by : dump_MA
call :
*/

void dump_listMH()
{
  struct mobile * mob;
  struct corr 	* ch;

  FORALLMAMH(mob)
    {
      logtrace("\t\thome       : %s",inet6pr(&mob->home));
      logtrace("\t\tcareof     : %s",inet6pr(&mob->careof));
      logtrace("\t\tVCOA       : %s",inet6pr(&mob->VCOA));
      if((MA_op == 2)&&(mob->HomeAgent != NULL))
      logtrace("\t\thome agent : %s",inet6pr(&mob->HomeAgent->addr));
      logtrace("\t\trefresh    : %d",mob->refresh);
      logtrace("\t\tlife       : %d",mob->lifetime);
      logtrace("\t\tseq nb     : %d",mob->seq);
      logtrace("\t\ttimer      : %x",mob->position_timer);
      logtrace("\t\tproxy      : %x",mob->ap);
      logtrace("\t\t****> correspondent nodes' list");  
      FORALLMHCORR(mob,ch)
      {
        logtrace("\t\t\tCH address : %s",inet6pr(&ch->addr));  
        logtrace("\t\t\tCH flags   : %x",ch->flags);  
        logtrace("\t\t\tCH seq nb  : %d",ch->seq);  
        logtrace("\t\t\tCH life    : %d",ch->lifetime);  
	if(ch->type==VCOA_NEEDED)
	{
    	    logtrace("\t\t\tCH careof  : %s",inet6pr(&ch->VCOA));  
	    logtrace("\t\t\tCH type    : VCOA");  
	}
	else
	{
    	    logtrace("\t\t\tCH careof  : %s",inet6pr(&ch->careof));  
	    logtrace("\t\t\tCH type    : PCOA");  
	}
        logtrace("\t\t\t****> next correspondent node");  
      }
      logtrace("\t\t--------------------------------------> next mobile node");
    }
}

/*
print the list of the base station

called by : dump_MA
call :
*/
void dump_listBS()
{
    struct BaseStation * bs;
    FORALLBS(bs)
    {
	logtrace("\tBS 	   	: %s",inet6pr(&bs->addr));
    }
    
}


void dump_listMA()
{
    struct MobilityAgent * ma;
    FORALLMA(ma)
	logtrace("\tMA		: %s|%d",inet6pr(&ma->addr),ma->charge);
}
#endif

#ifdef uMIPBS
/*
ndpd-router is running in debug mode, the user presses CTRL+C,
we have to dump all interesting data.
here the router is a Base Station

called by : main
call :
*/
void dump_BS()
{
    logtrace("\tThis router is a Base Station");
    logtrace("\tLocal MA : %s mode %d",inet6pr(&MA_info.MA_address),MA_info.mode);
}
#endif

#ifdef UHMMA
/*
ndpd-host is running in debug mode, the user presses CTRL+C,
we have to dump all interesting data.

print the mobility mode, the actual sequence number, 
the home agent address, the mobility agent address,
the previous mobility agent address
we print all the timer pointers, when a timer expires we affect 0 to its pointer
so by printing the timer pointer we show the active timers.

called by : main
call :
*/
void dump_uhmma()
{
  logtrace("\n\tUHMMA Information\n");
  logtrace("\tCurrent Mobility Mode : %d",global.mobility_mode);
  logtrace("\tInitial Mobility Mode : %d",global.mobility_mode_init);
  logtrace("\tCurrent PCOA          : %s",inet6pr(&global.HA->careof));
  logtrace("\tCurrent VCOA          : %s",inet6pr(&global.VCOA));
  logtrace("\tSequence Counter      : %d",global.seq_num);
  logtrace("\tMobility Agent RTADV  : %s", inet6pr(&global.MA_readed));
  logtrace("\tMobility Agent        : %s", inet6pr(&global.MA_address));
  logtrace("\tPrevious MAgent       : %s", inet6pr(&global.MA_old));
  logtrace("\told position timer    : %x",global.uMIP_timer);
  logtrace("\tretransmission timer  : %x",global.reemis_timer);
  logtrace("\thome sweet home timer : %x",global.back_timer);
  logtrace("\tsite prefix length    : %d",global.site_len);
  logtrace("\tmax number of retrans : %d",global.reg_ret_max);
}

#endif
	
