#ifdef uMIP

#include "ndpd.h"
#include "../ndpd-mobility/mobility.h"
#include "../ndpd-uhmma/uhmma.h"
#include <net/bpf.h>

static char bbuf[BUFSIZ];
static struct bpf_insn bi[] = 
{
    BPF_STMT(BPF_LD  +  BPF_B    +  BPF_ABS,  20),
    BPF_JUMP(BPF_JMP +  BPF_JEQ  +  BPF_K,    0x29,  0, 3 ),
    BPF_STMT(BPF_LD  +  BPF_B    +  BPF_ABS,  78),
    BPF_JUMP(BPF_JMP +  BPF_JEQ  +  BPF_K,    0xff, 1 , 0 ),
    BPF_STMT(BPF_RET +  BPF_K, 94),
    BPF_STMT(BPF_RET +  BPF_K, 0),
};

static struct bpf_program bp = { 6, bi };

/*static struct bpf_insn bi[] = 
{
    BPF_STMT(BPF_RET +  BPF_K, 94),
};

static struct bpf_program bp = { 1, bi };*/


/*
open the bpf on the virtual interface to detect encapsulated packets 
(i.e. new cn for managed mh).
the program we use to detect encapsulated packet is :

if P(20) == 0x29 then return packet
		 else nothing

P is the received packet. P(20) is the 20st byte of the packet, it is
the IPv6 next header field. 0x29 is encapsulated header.

the virtual network structure must be initialized before the call of this
function !

called by : read_gateway
call :
*/
char open_bpf()
{
	struct if_laddrreq ifr6;
	struct ifreq ifr;
	struct sockaddr_in6 *sin;
	char dname[12];
	int d, unit = 0;
	u_int val;

retry:
    /* try to open */
	(void) snprintf(dname, sizeof(dname) - 1, "/dev/bpf%d", unit++);
	if ((socket_bpf = open(dname, O_RDONLY)) < 0) 
	{
		if (errno == EBUSY)
			goto retry;
		logperror("open (BPF)");
		return 0;
	}
    /* initialize the buffer length */
	val = BUFSIZ;
	if (ioctl(socket_bpf, BIOCSBLEN, &val) < 0)
		logperror("ioctl (BIOCSBLEN)");
    /* start the sniffing NOW ! */
	val = 1;
	if (ioctl(socket_bpf, BIOCIMMEDIATE, &val) < 0)
		logperror("ioctl (BIOCIMMEDIATE)");
    /* attach the bpf to physical interface */
	bzero(&ifr, sizeof(ifr));
	strncpy(ifr.ifr_name, VirtualNetwork.root, IFNAMSIZ);
	if ((ioctl(socket_bpf, BIOCSETIF, &ifr) < 0) )/* && verboseU */
		logperror("ioctl (BIOCSETIF)");
    /* program the bpf */
	if (ioctl(socket_bpf, BIOCSETF, &bp) < 0)
		logperror("ioctl (BIOCSETF)");
	if (verboseU)
		logtrace("\tuMIP ; open BPF on %s linked on %s", 
			    dname,VirtualNetwork.root);
    return 1;
}

/*
close the bpf opened for the cn detection.

called by : mobility_cleanup
call :
*/
void close_bpf()
{
    if(verboseU)
	logtrace("\tuMIP ; close the bpf");	
    if (socket_bpf < 0)
		return;
	(void) ioctl(socket_bpf, BIOCFLUSH, NULL);
	(void) close(socket_bpf);
	socket_bpf = -1;
}
/*
receive an encapsulated packet on the virtual network.
we analyze the packet to determine the mobile host and 
add the cn in the list of cn of the mh.
we check if this cn is in the local domain, to send it a BU
with the PCOA or the VCOA.

called by : main
call : ad_corr, send_bnd_upd
*/
void recv_bpf()
{
	struct bpf_hdr *hp;
	struct ip6_hdr *ip,*eip;
	struct mobile * mob;
	struct corr * cop;
	u_int *afp;
	char *p,*ptr;
	int cc,ii;

	cc = read(socket_bpf, bbuf, sizeof(bbuf));

	if(verboseU)
	    logtrace("\tuMIP ; intercepted an encapsulated packet");
	/* for testing */
/*	if(verboseU)
	{
	    struct in6_addr *src,*dst;
	    src = (struct in6_addr *)bbuf+22;
	    dst = (struct in6_addr *)bbuf+38;
	    logtrace("\tsrc = %s",inet6pr(src));
	    logtrace("\tdst = %s",inet6pr(dst));
	}
	return;
*/
	if (cc < 0) 
	{
		logperror("read (BPF)");
		return;
	}
	if (cc < sizeof(*hp)) 
	{
		if (verboseU)
			logerr("recv_bpf: very short (%d) read", cc);
		return;
	}

	p = bbuf;

	for (;;) 
	{
		hp = (struct bpf_hdr *)p;
		afp = (u_int *)(p + hp->bh_hdrlen);
		ip = (struct ip6_hdr *)(afp + 1);
		ptr = (char *)ip ;
		ptr+=50;
		eip = (struct ip6_hdr *)ptr;

/*		if (*afp != AF_INET6)
			return;

		if ((ip->ip6_flow & IPV6_FLOWINFO_VERSION) != IPV6_VERSION)
			return;
*/
		if(SAME_ADDR6(eip->ip6_src, MA_address))
			goto next;
		logtrace("\tuMIP ; eip %s for %s", 
		    inet6pr(&eip->ip6_src),inet6pr(&eip->ip6_dst));
		
		/* find the mobile host */
		FORALLMAMH(mob)
		    if(SAME_ADDR6(eip->ip6_dst, mob->home))
			break;
		if(mob==NULL)
		{
		    if(verboseU)
			logtrace("\tuMIP ; this mobile host is not registered here");
		    goto next;
		}
		/* the mobile runs in mode 1 */
		if(mob->CH.tqh_first == NULL)
		    return;
		/* find the correspondent node */
		FORALLMHCORR(mob,cop)
		    if(SAME_ADDR6(eip->ip6_src, cop->addr))
			break;
		if(cop != NULL)
		{
		    if(verboseU)
			logtrace("\tuMIP ; this cn is allready registered");
		}
		else
		{
		    if(verboseU)
			logtrace("\tuMIP ; this cn is a new one");
		    cop = add_corr(&eip->ip6_src,mob);
		}
		send_bnd_upd(cop);
next:
		p += BPF_WORDALIGN(hp->bh_hdrlen + hp->bh_caplen);
		if (p >= bbuf + cc)
			return;
	}
}




#endif

