#ifdef uMIP
#include "ndpd.h"
#include "../ndpd-mobility/mobility.h"

#include "../ndpd-uhmma/uhmma.h"

void   build_list(struct mobile * ,int, char * );
void   store_list(struct mobile *, u_char *,int);
void   send_BU(struct mobile *);

/*
the MA receive a cn list in a back packet.
we have to search the mobile host(by using the WaintingBack queue)
or maybee the mh is already registered because it was in a MIP domain
and it sent an Ureg + Cn list

then we can add all the correspondents in the mobile's cn list,
update the list with the new PCOA and VCOA and the correct seq# and 
finally send BU to CH.

if the status of the pkt is not 0, then we have to transmit this
status to the mobile by using a BACK and to remove the mobile host
from the list.

called by : recv_bnd_ack
call : store_list, send_BU, kill_mobile, send_bnd_ack, 
*/

void recv_cn_list(src, cp, len)
struct in6_addr *src;	/* ip header */
u_char *cp;		/* destination option */
int len;		/* option length */
{
    u_char status;
    u_short seq;
    u_int life, refresh;
    u_char type;
    u_char length;
    int pad;
    struct in6_addr home;
    struct WBACK * wb;
    struct in6_addr * svg = src;
    struct mobile * mob;
    

    /* decode the BACK */
    GET8(status, cp);	
    GET16(seq, cp);
    GET32(life, cp);
    GET32(refresh, cp);
    
    /*----------------------------------------
	the smooth handoff sucks
    ----------------------------------------*/
    if (status == OPT6_BNDA_CN_LIST_KO) 
    {
	if(verboseU)
	    logtrace("\tuMIP ; smooth handoff failed, tell it to the MH");
	/* recover the home address */	
	GET8(type,cp);
	GET8(length,cp);
	if ((type != SUB_OPT6_CN) ||(length !=22))
	{
	    if(verboseU)
		logtrace("\tuMIP ; receive a buggy cn list");
	    return;
	}
	bcopy(cp+6,&home,sizeof(struct in6_addr));
	FORALLWB(wb)
	{
	    /* the src matches */
	    if(SAME_ADDR6(wb->MA,*svg))
		if(SAME_ADDR6(wb->MH->home, home))
		    goto mistake;
	}
	if(verboseU)
	    logtrace("\tuMIP ; receive a buggy cn list but no wback structure");
	return;
mistake:
	/* stop the retransmission timer */
	DESTROY_TIMER(wb->tm);
	/* binding ack */
	wb->MH->seq = seq;
	send_bnd_ack(wb->MH, OPT6_BNDA_REFUSED, wb->MH->refresh);
	/* we have to add 1 to the charge because kill_mobile will remove 1 because no ch is registered in the list */
	charge ++;
	kill_mobile(wb->MH);
	/* remove the WBACK structure */
	free(wb->pkt);
	TAILQ_REMOVE(&waitingBack,wb,wb_list);
	free(wb);
	return;
    }

    /*----------------------------------------------
	the smooth handoff successfully completed
    ----------------------------------------------*/

    /* decode the cn list */
    GET8(type, cp);
    if(type != SUB_OPT6_CN)
    {
	if(verboseU)
	    logtrace("\tuMIP ; received a cn list with a strange sub option type");
	return;
    }
    GET8(length,cp);
    /* todo : check the length */
    GET16(pad,cp);
    GET32(pad,cp);
    bcopy(cp,&home,sizeof(struct in6_addr));

    if(verboseU)
	logtrace("\tuMIP received a cn list from %s seq#%d",inet6pr(src),seq);
    /* find the waiting back */
    FORALLWB(wb)
	/* the src matches */
	if(SAME_ADDR6(wb->MA,*svg))
	    if(SAME_ADDR6(wb->MH->home, home))
		goto found;
    /* find the swapping mobile host */
    FORALLMAMH(mob)
	if(SAME_ADDR6(mob->home, home))
	    goto swappy;
    if(verboseU)
	logtrace("\tuMIP ; receive a no expected cn list");
    return;

found:
    /* chekc seq# */
    if(seq!=wb->MH->seq)
    {
	if(verboseU)
	    logtrace("\tuMIP ; receive an expected cn list with a too small seq#%d %d",
			seq,mob->seq);
	return;
    }
    charge +=3;
    update_MA(&MA_address,charge);
    /* stop the retransmission timer */
    DESTROY_TIMER(wb->tm);
    /* list stuff */
    cp+=16;
    store_list(wb->MH,cp,length);
    send_BU(wb->MH);
    /* start the life timer for this mobile */
    wb->MH->position_timer=new_timeout(life, 0, (func_to)kill_mobile,wb->MH);
    send_bnd_ack(wb->MH, OPT6_BNDA_ACCEPT, wb->MH->refresh);
    /* remove the WBACK structure */
    free(wb->pkt);
    TAILQ_REMOVE(&waitingBack,wb,wb_list);
    free(wb);
    return;
swappy:
    /* chekc seq# */
    if(seq!=mob->seq)
    {
	if(verboseU)
	    logtrace("\tuMIP ; receive an expected cn list with a too small seq#%d %d",
			seq,mob->seq);
	return;
    }
    /* do not add charge here because it is done in recv_uhmma_reg */
    /* list stuff */
    cp+=16;
    store_list(mob,cp,length);
    send_BU(mob);
    /* re start the life timer for this mobile */
    RESTART_TIMER(mob->position_timer,life,kill_mobile,mob);
    /* acknowledgment */
    send_bnd_ack(mob, OPT6_BNDA_ACCEPT, temp.refresh);
    return;
}

/*
store the list received in the back sub option in the mobile
cn list.
while inserting the structures in the list, we update the 
addresses fields and check if the cn is in the local domain,
this is useful for sending bu with pcoa or vcoa.

! : don't forget to reset the AH flag on the HA structure

called by : recv_cn_list
call : same_prefix
*/
void store_list(mob,ptr,len)
struct mobile * mob;	/* the mobile host */
u_char * ptr;		/* the CH list */
int len;		/* option length */
{
    u_char * par = ptr;
    struct corr4list tmp;
    int nbCN = (len -6 -16)/sizeof(struct corr4list);
    int ii;
    
    /* initialize the CH list */
    TAILQ_INIT(&mob->CH);

    for (ii=0;ii<nbCN;ii++)
    {
	struct corr * cop;
	bcopy(par,&tmp,sizeof(struct corr4list));
	if(Calloc(cop,corr) == NULL)
	{
	    if(verboseU)
		logtrace("\tuMIP ; problem during cn list registration");
	    return;
	}
	/* add the CH structure */
	if(ii ==0)
	    TAILQ_INSERT_HEAD(&mob->CH, cop, cr_list);
	else
	    TAILQ_INSERT_TAIL(&mob->CH, cop, cr_list);

	COPY_ADDR6(tmp.addr,cop->addr);
	cop->type=tmp.type;
	cop->flags=tmp.flags | 0x20; /* C flag means coa in the bu */
	cop->plen=tmp.plen;
	cop->seq=tmp.seq;
	cop->lifetime=tmp.lifetime;
/*	cop->retcnt=tmp.retcnt ;*/

	/* update the ch structure with the new addresses */
	COPY_ADDR6(mob->careof,cop->careof);
	COPY_ADDR6(mob->home,cop->home);
	COPY_ADDR6(mob->VCOA,cop->VCOA);
	/* where is the ch ?
	    the HA always need vcoa
	    initialize the HA pointer
	*/
	if(cop->flags & OPT6_BNDU_HR) 
	{
	    mob->HomeAgent = cop;
	    cop->type = VCOA_NEEDED;
	    /* the last MA removes the A flag if it sent several home
	    registrations (to prevent the mobile host to receive not 
	    expected back) */
	    cop->flags |= OPT6_BNDU_ACK;
	}
	else
	    if(same_prefix(&cop->addr,&mob->careof,site_len)==0)
		cop->type = VCOA_NEEDED;
	    else
		cop->type = NO_VCOA_NEEDED;
	/* synchronize the seq# */
	if(mob->seq > cop->seq)
	    cop->seq = mob->seq -1;/* -1 because send_bnd_upd does ++ */
	/* if  HA */
	if(cop->flags  & OPT6_BNDU_HR)
	    if(mob->seq < cop->seq)
		mob->seq = cop->seq;
	
	/* next one */
	par+=sizeof(struct corr4list);
    }
}

/*
build and send the correspondents' list of a mobile node
to its new MA

called by : recv_smooth
call : build_list, send_kernel
*/
void send_cn_list(mop, MA_src)
struct mobile * mop;		/* the mobile host */
struct in6_addr * MA_src;	/* the MA destination address */
{
	int i,d, cc, olen;
	struct corr * cop;
	int cnnb = 0;
	char * list;
	int buffer_size;
	int byte_8_length;
	int sub_option_length ;
	int option_length;
	
	if(verboseU)
	    logtrace("\tuMIP ; will send cn list");
	    
	/* find the number of cn */
	FORALLMHCORR(mop, cop)
	    cnnb++;
	
	list = (char *)malloc (cnnb * sizeof(struct corr4list));
	if(list == NULL) 
	{
	    if(verboseU)
		logtrace("\tuMIP ; not enough memory to send corr list");
	    return;
	}
	build_list(mop,cnnb,list);
	sub_option_length = cnnb * sizeof(struct corr4list)+6+16;
			/* don't forget the two first bytes of the sub option (pad for alignment)
			the length and the size must not be included
			don't forget the home address*/
	option_length = 11+sub_option_length+2;\
			/* size of the BACK option , include the sub-option
			don't forget the first line of the sub option
			and the home address*/
	buffer_size = option_length+5;
			/* size of the option + the type + the length field
			    and don't forget the alignment
			*/
	byte_8_length = buffer_size/8 - 1;
			/* 8byte size ; a corr4list structure is 32byte long
			don't forget the first line of the sub option
			and the home address*/
	/* fill a destination option header with the binding ack */
    {
	char babuf[buffer_size];
	register u_char *cp;
	
	cp = (u_char *)babuf;

	SET8(IP6_NHDR_NONH, cp);
	SET8(byte_8_length, cp);   
	SET8(OPT6_PAD_0, cp);
	SET8(OPT6_BINDING_ACK, cp);
	SET8(option_length, cp);
	SET8(OPT6_BNDA_CN_LIST_OK, cp);/* status */
	SET16(mop->seq, cp);
	SET32(mop->lifetime, cp);
	SET32(temp.refresh, cp);
	/* sub option with cn list */
	SET8(SUB_OPT6_CN,cp);
	SET8(sub_option_length , cp);

	SET16(0,cp);
	SET32(0,cp);
	SETADDR(mop->home, cp);
	bcopy(list,cp,cnnb*32);

	if(send_kernel("send cn list",babuf,buffer_size,&MA_address,MA_src,0,0)==1)
	    if(verboseU)
		logtrace("\tuMIP ; send cn list to %s seq#%d",
			inet6pr(MA_src),mop->seq);
    }
	/* free the list */
	free(list);
}
/*
build the correspondents' list to send in the BACK packet
(it is a list of corr4list structures) 
by using the correspondents' list of the mobile host
(it is a list of corr structures) 

called by : send_cn_list
call :
*/
void build_list(mop,nb,list)
struct mobile * mop;	/* the mobile host */
int nb;			/* number of CH */
char * list;		/* the list to fill */
{
	struct corr4list tmp;
	struct corr * cop;
	char * l = list;
	
	FORALLMHCORR(mop,cop)
	{
	    COPY_ADDR6(cop->addr,tmp.addr);
	    tmp.type = cop->type;
	    tmp.flags = cop->flags;
	    tmp.plen = cop->plen;
	    tmp.seq = cop->seq;
	    tmp.lifetime = cop->lifetime;
/*	    tmp.retcnt = cop->retcnt;*/
	    bcopy(&tmp,l,sizeof(struct corr4list));
	    l+=sizeof(struct corr4list);
	}
}

/*
we had store the list of new cn for a new mobile, due to a smooth handoff
we are sure that the mobile host performed an intra site movement 
or a major move
now we have to send BU to this cn to tell them the new position of the mobile ;
VCOA for HA and external cn
PCOA for local cn

called by : recv_cn_list
call : send_bnd_upd, send_home_reg
*/
void send_BU(MH)
struct mobile * MH;	/* the mobile host */
{
    struct corr * CH;
    FORALLMHCORR(MH, CH)
    {
	if(CH->flags  & OPT6_BNDU_HR)
	    send_home_reg(CH);
	else
	    send_bnd_upd(CH);
    }
}
#endif
