/* 
forwarding is the action to forward packets for a mobile host witch was 
registered and it's gone in an other domain, 
the two MA communicate to perform the forwarding of packet addressed to the
old VCOA to the new one.
 */
#ifdef uMIP

#include "ndpd.h"
#include "../ndpd-mobility/mobility.h"
#include "../ndpd-uhmma/uhmma.h"

/*
this function send a forward packet,
this is a BU with the VCOA of the mobile and a specific flag FORWARD_FLAG
this packet is send to the previous MA of the mobile host
the previous MA has to forward packets to the new position of the mobile
(ie the vcoa in the BU)

called by : recv_uhmma_reg
call : send_kernel
*/
void send_forward(MAold,mh)
struct in6_addr * MAold;	/* the address of the previous MA */
struct mobile * mh;		/* the mobile host */
{
	int byte_length = 48;	/* must be a 8multiple */
	int byte_8_length = (byte_length -8) / 8;
	char bubuf[byte_length];
	register u_char *cp;

	/* build the packet */
	cp = (u_char *)bubuf;

	SET8(IP6_NHDR_NONH, cp);
	SET8(byte_8_length, cp);
	/* DO BU  */
	SET8(OPT6_BINDING_UPD, cp);
	SET8(24, cp);
	SET8(FORWARD_FLAG|0x20, cp);/* coa in the BU */
	SET8(0, cp);
	SET16(mh->seq, cp);
	SET32(temp.forward, cp);/* PURSUIT_TIMEOUT < DEFAULT_MH_LIFETIME */
	SETADDR(mh->VCOA, cp);

	SET8(OPT6_PAD_0, cp);
	SET8(OPT6_PAD_0, cp);

	/* DO H@ */
	SET8(OPT6_HOME_ADDR, cp);
	SET8(16, cp);
	SETADDR(mh->home, cp);

	/* send */
	if(send_kernel("send forward",bubuf,byte_length,&ME,MAold,0,0) == 0)
	    return;
	if (verboseU) 
		logtrace("\tuMIP ; send forward to %s for %s seq#%d",
				 inet6pr(MAold),inet6pr(&mh->home),
				 mh->seq);
}

/*
this MA received a forward packet, a bu to ask to redirect packets 
addressed to a mobile host
this mh is gone and will not send old position packet
so we restart the timer for this mh for the very last time

a forward bu can only be received if the MA runs mode 1 (else the BU
should have be a smooth BU).

called by : recv_destopt
call : mobile_forward
*/
void recv_forward(ip, cp,MA_src)
struct ip6_hdr *ip;		/* the ip header of the forward BU */
u_char *cp;			/* the option */
struct in6_addr *MA_src;	/* the sender */
{
	struct in6_addr *careof;
	register struct mobile *mop;
	u_char flags, plen, status = OPT6_BNDA_ACCEPT;
	u_short seq;
	u_int life;
	struct in6_addr *home;

	/* decode the binding update */
	home = &ip->ip6_src;
	GET8(flags, cp);
	GET8(plen, cp);
	GET16(seq, cp);
	GET32(life, cp);
	careof = (struct in6_addr *)cp;

	if (verboseU) {
		logtrace("\tuMIP ; received forward from (MA) %s for (home) \n\t%s seq#%d",
			 inet6pr(&ip->ip6_src), inet6pr(home),seq);
	}

	/* look for a mobile entry */
	FORALLMAMH(mop)
	    if (IN6_ARE_ADDR_EQUAL(home, &mop->home))
		break;
	/* do the stuff for forwarding */
	/* mobile_forward check if mop==null and sends BACK if needed */
	mobile_forward(mop,seq,life,MA_src,careof);
}
#endif