/*
This functions are used by a mobile host to send
binding updates and home registrations.
The mobility agent uses them to send binding updates
and home registration on behalf of the mobile hosts.
The packet is not the same, we add the COA in the
binding update option field. The packet grows by 16
and the flag C is set. 
For binding updates we have to check to location of
the correspondent node. If it is in the site, we add
the PCOA in the BU, or the VCOA if it an external CH.

we copy the functions, and hack them instead of hacking the existing functions
witch are only compiled on mobile host.
*/

#ifdef uMIP
#include "ndpd.h"
#include "../ndpd-mobility/mobility.h"
#include "../ndpd-uhmma/uhmma.h"

void send_bnd_upd(cop)
struct corr *cop;	/* the CN destination */
{	
	register u_char *cp;
	char bubuf[48];

	/* update the seq# for this ch */
	cop->seq ++;

	if (verboseU) 
	{
		logtrace("\tuMIP ; will send binding update");
		logtrace("\tuMIP ; addr=%s", inet6pr(&cop->addr));
		logtrace("\tuMIP ; home=%s", inet6pr(&cop->home));
		logtrace("\tuMIP ; flags=%x [%s] life=%u seq=%d retcnt=%d",
			 cop->flags, bnd_upd_flg(cop->flags),
			 cop->lifetime, cop->seq, cop->retcnt);
	}

	/*
	 * Fill a destination option header with the binding update
	 * and a home address options.
	 */

	cp = (u_char *)bubuf;

	SET8(IP6_NHDR_NONH, cp);
	SET8(5, cp);/* we add the coa in the BU, \
		      so the do length += 2 8-byte units */
	SET8(OPT6_BINDING_UPD, cp);
	SET8(24, cp);			/* the coa length is 16 bytes */
	SET8(cop->flags, cp);	/* included C flag -> coa in the BU */
	SET8(0, cp);
	SET16(cop->seq, cp);
	SET32(cop->lifetime, cp);
	if (cop->type == VCOA_NEEDED)
	{
	    SETADDR(cop->VCOA, cp);
	    if (verboseU) 
		logtrace("\tuMIP ; care-of=%s", inet6pr(&cop->VCOA));
	}
	else if (cop->type == NO_VCOA_NEEDED)
	{
	    SETADDR(cop->careof,cp);
	    if (verboseU) 
		logtrace("\tuMIP ; care-of=%s", inet6pr(&cop->careof));
	}
	else if (cop->type == HOME_NEEDED)
	{
	    SETADDR(cop->home,cp);
	    if (verboseU) 
		logtrace("\tuMIP ; care-of=%s", inet6pr(&cop->home));
	}

	SET8(OPT6_PAD_0, cp);
	SET8(OPT6_PAD_0, cp);

	SET8(OPT6_HOME_ADDR, cp);
	SET8(16, cp);
	SETADDR(cop->home, cp);

	/* send */
	if(send_kernel("send binding update",bubuf,sizeof(bubuf),
			&ME,&cop->addr,0,0)==0)
	    return;

	if (verboseU) 
		logtrace("\tuMIP ; sent binding update");
    /* timer stuff */
    if(cop->lifetime == 0)
	return;
    if(cop->tp == NULL)
	cop->tp = new_timeout(cop->lifetime-10,0,(func_to)send_bnd_upd,cop);
    else
	if(cop->tp->status == -1) /* it means that the timer has expired */
	    set_timeout(cop->tp,cop->lifetime-10,0,(func_to)send_bnd_upd,cop);
	else
	    RESTART_TIMER(cop->tp,cop->lifetime-10,send_bnd_upd,cop);
}

void send_home_reg(agp)
struct corr *agp;	/* the HA destination */
{
	register u_char *cp;
	char bubuf[48];

	/* update the seq# for this ch */
	agp->seq ++;
	/*
	 * Fill a destination option header with the home registration
	 * and the home address options.
	 */

	cp = (u_char *)bubuf;

	SET8(IP6_NHDR_NONH, cp);
	SET8(5, cp);			/* we add a coa in the BU, 
		                          so the do length += 2 8-byte units */

	SET8(OPT6_BINDING_UPD, cp);
	SET8(24, cp);			/* the coa length is 16 bytes */
	SET8(agp->flags, cp);	/* included C flag -> coa in the BU */

	SET8(0, cp);			/* was agp->plen */
	SET16(agp->seq, cp);
	SET32(agp->lifetime, cp);
	if (agp->type == VCOA_NEEDED)
	{
	    SETADDR(agp->VCOA, cp);
	}
	else if (agp->type == HOME_NEEDED)
	{
	    SETADDR(agp->home,cp);
	}
	else
	{
	    if(verboseU)
		logtrace("\tuMIP ; problem during sending of home registration");
	    return;
	}
	SET8(OPT6_PAD_0, cp);
	SET8(OPT6_PAD_0, cp);

	SET8(OPT6_HOME_ADDR, cp);
	SET8(16, cp);
	SETADDR(agp->home, cp);

	/* send */
	if(send_kernel("send home registration",bubuf,sizeof(bubuf),
			&ME,&agp->addr,0,0)==0)
	    return;

	if (verboseU) 
	{
		logtrace("\tuMIP ; sent home registration (add) to ");
		logtrace("\tuMIP ; addr=%s", inet6pr(&agp->addr));
		logtrace("\tuMIP ; home=%s", inet6pr(&agp->home));
		logtrace("\tuMIP ; flags=%x [%s] life=%u seq=%d retcnt=%d",
			 agp->flags, bnd_upd_flg(agp->flags),
			 agp->lifetime, agp->seq, agp->retcnt);
		logtrace("\tuMIP ; care-of=%s", inet6pr(&agp->VCOA));
	}

    /* the very first time we send a home registration, we set the A flag
       because the mobile host is waiting for this back.
       but for the retransmission, we don't have to set this flag.
       the mobile host does not know if a home registration is performed,
       so it will not waiting for a back.
       without this flag, the HA will not acknowledge
       
       note : if the mobile host performs a macro move, the new ma receives 
       the cn list of the mh. it has to reset the A flag for sending the first 
       HR.
     */

    /* timer stuff 
	    - the very first time the pointer to the timer's structure is NULL
	    - if the timer expires, the timer can be reused 
	    - if the ma decides to resend a home registration before the end
	      of the timer, we can reused it, but kill it first
       note that if the lifetime of this ch is 0, no need to restart the timer.
    */
    if(agp->lifetime == 0)
	return;
    if(agp->tp == NULL)
    {
	agp->tp = new_timeout(agp->lifetime-10,0,(func_to)send_home_reg,agp);
	agp->flags &=~OPT6_BNDU_ACK;
    }
    else
	if(agp->tp->status == -1) /* it means that the timer has expired */
	    set_timeout(agp->tp,agp->lifetime-10,0,(func_to)send_home_reg,agp);
	else
	    RESTART_TIMER(agp->tp,agp->lifetime-10,send_home_reg,agp)
/* todo : check if lifetime - 10 > 0 */
}

/*

we rewrite this function to send bnd ack directly to the mobile host's position
instead of the home with a rt header.
THIS FUNCTION IS CALLED ONLY BY MA.
no rt hdr is needed here.

*/



#endif
