#ifdef uMIP

#include "ndpd.h"
#include "../ndpd-mobility/mobility.h"
#include "../ndpd-uhmma/uhmma.h"

static char smooth_retrans;

void resend_smooth(struct WBACK *);

/*
send a smooth handoff packet, ie a packet with a BU destination 
option, a H@ destination option and the flag C

start a timer for restansmission.

create a record in waitingBack list, to back up smoth handoff 
information.

called by : recv_uhmma_reg
call : new_timeout
*/
void send_smooth(dest, mob)
struct in6_addr * dest;		/* destination address (ie old MA) */
struct mobile * mob;		/* mobile host */
{
	struct WBACK * wback;
	register u_char *cp;
	int byte_length = 48;
	int byte_8_length = (byte_length -8)/8;
	char * bubuf = malloc(byte_length);

	if(
	    (Calloc(wback, WBACK) == NULL)||
	    (bubuf == NULL)
	  )
	{
	    logerr("not enough memory to procede with the smooth handoff");
	    return;
	}
	/*
	 * Fill a destination option header with the binding update
	 * and a home address options.
	 */
	cp = (u_char *)bubuf;

	SET8(IP6_NHDR_NONH, cp);
	SET8(byte_8_length, cp);

	SET8(OPT6_BINDING_UPD, cp);
	SET8(24, cp);
	SET8(SMOOTH_FLAG|0x20, cp);	/* C flag */
	SET8(0, cp);
	SET16(mob->seq, cp);
	SET32(temp.forward, cp);/* PURSUIT_TIMEOUT < DEFAULT_MH_LIFETIME */
	SETADDR(mob->VCOA, cp); /* virtual COA */

	SET8(OPT6_PAD_0, cp);
	SET8(OPT6_PAD_0, cp);

	SET8(OPT6_HOME_ADDR, cp);
	SET8(16, cp);
	SETADDR(mob->home, cp); /* home address */

	/* send */
	if(send_kernel("send smooth handoff",bubuf,byte_length,
			&MA_address,dest,0,0) == 0)
	{
	    free(bubuf);
	    free(wback);
	    return;
	}
	if (verboseU) 
	    logtrace("\tuMIP ; sent smooth handoff to %s seq#%d",
			inet6pr(dest),mob->seq);
    /* waiting BACK */
    wback->MH = mob;
    COPY_ADDR6(*dest, wback->MA);
    wback->pkt = bubuf;
    wback->pkt_size = byte_length;
    TAILQ_INSERT_HEAD(&waitingBack,wback,wb_list);
    /* retransmission */
    wback->tm=new_timeout(temp.smooth, 0, (func_to)resend_smooth, wback);
    wback->retrans = 0;
}

/*
resend the smooth handoff packet when the timer expire
this packet is stored in memory, 
the pointer on it is in the structure wback

restart the retransmission timer, witch is in the structure

called by : run_timeout
call : set_timeout
*/
void resend_smooth(wback)
struct WBACK * wback;	/* the structure witch contains the data for this smooth */
{
	char * upd;

	if(wback->retrans ++ >= max_retrans)
	{
	    /* stop the retransmission timer */
	    DESTROY_TIMER(wback->tm);
	    /* binding ack */
	    send_bnd_ack(wback->MH, OPT6_BNDA_REFUSED, wback->MH->refresh);
	    kill_mobile(wback->MH);
	    /* remove the WBACK structure */
	    free(wback->pkt);
	    TAILQ_REMOVE(&waitingBack,wback,wb_list);
	    free(wback);
	    return;
	}
    /* upgrade seq# */
    upd = wback->pkt + 6;
    wback->MH->seq++;
    SET16(wback->MH->seq, upd);
    

	/* send */
	if(send_kernel("resend smooth",wback->pkt,wback->pkt_size,&MA_address,
			&wback->MA,0,0) == 0)
	    return;
	if (verboseU) 
		logtrace("\tuMIP ; re sent smooth to %s seq#%d retrans#%d", 
		    inet6pr(&wback->MA),wback->MH->seq,smooth_retrans);
    /* restart the retransmission timer */
    set_timeout(wback->tm, temp.smooth, 0, (func_to)resend_smooth, wback);
}



/*
receive a smooth handoff BU packet
change the tug to the mobile's position
restart mobile's timer
and send BACK with correspondents' list to the MA src.

we stop sending BU on behalf of this MH but we don't remove the list
of CH, we simply kill all timer, because the smooth handoff could suck
so the MA will resend the smooth BU, we must still have the list to 
send it.

called by : recv_destopt
call : send_cn_list, 
*/
void recv_smooth(ip, cp, MA_src)
struct ip6_hdr *ip;		/* the ip header */
u_char *cp;			/* the first option = BU */
struct in6_addr *MA_src;	/* the sender */
{
    struct in6_addr * home;
    struct in6_addr * careof;
    u_char flags, plen, status = OPT6_BNDA_ACCEPT;
    u_short seq;
    u_int life;
    struct mobile * mop;
    struct corr * ch;

    /* decode the sub option */
    home = &ip->ip6_src;
    GET8(flags, cp);
    GET8(plen, cp);
    GET16(seq, cp);
    GET32(life, cp);
    careof = (struct in6_addr *)cp;

    if (verboseU) 
    	logtrace("\tuMIP ; received smooth from (MA/MH) %s for (home) %s seq#%d",
		 inet6pr(MA_src), inet6pr(home),seq);

    /* look for a mobile entry */
    FORALLMAMH(mop)
	if (SAME_ADDR6(*home, mop->home))
		break;

    /* pursuit stuff */
    if (mobile_forward(mop, seq,life,MA_src,careof) == 0)
    {
	register u_char *cp;
	char babuf[40];
	if(verboseU)
	    logerr("\tuMIP ; received a smooth BU for an unregistered mobile ");

	/* fill a destination option header with the binding ack */
	/* we have to put the H@ in the BACK to identify the MH */
	cp = (u_char *)babuf;

	SET8(IP6_NHDR_NONH, cp);
	SET8(4, cp);
	SET8(OPT6_PAD_0, cp);
	SET8(OPT6_BINDING_ACK, cp);
	SET8(35, cp);
	SET8(OPT6_BNDA_CN_LIST_KO, cp);
	SET16(seq, cp);
	SET32(life, cp);
	SET32(0, cp);
	SET8(SUB_OPT6_CN,cp);
	SET8(22, cp);
	SET16(0,cp);	/* pad */
	SET32(0,cp);	/* pad */
	SETADDR(*home,cp);

	/* send */
	if(send_kernel("send binding ack",babuf,sizeof(babuf),
			&MA_address,MA_src,0,0)==0)
	    return;
	if (verboseU) 
		logtrace("\tuMIP ; send binding ack [%d] to %s seq#%d",
				 status, inet6pr(MA_src),seq);

	return;
    }	
    /* build n send the cn list */
    send_cn_list(mop, MA_src);
    /* stop BU sending */
    FORALLMHCORR(mop,ch)
	DESTROY_TIMER(ch->tp);
}

#endif
