#ifdef UHMMA

#include "ndpd.h"
#include "../ndpd-mobility/mobility.h"
#include "../ndpd-uhmma/uhmma.h"

/*

a mobile can receive a lot of different BACK, from different protocols, 
so we just make a call to this function in recv_bnd_ack to keep it clear.
if the back is from uhmma protocol, we do what we have to do, and return 1,
it tells to recv_bnd_ack that the back have been .

we are going to check if this BACK contains interesting things for uhmma
if its needed code will be executed.

First check the mobility mode.

UHMMA Mode 2 :
Check if the packet is coming from the local
mobility agent or from the home agent.
If the mobile host performed an intra site movement
it needs only one BACK (from the mobility agent),
or two BACK for an inter site movement.
Then check the sequence number.
Then check the nbBACK. It could be 
-1 -> the mobile host is waiting for a back from the ma (micro movement)
 0 -> no back is needed
 1 -> the mobile host already received the back from the ma, 
      it is waiting for the ha
 2 -> the mobile host already received the back from the ha, 
      it is waiting for the MA
 3 -> the mobile host is waiting for 2 back (macro movement)
 20 -> the mobile host just changed is mobility mode from 2 to 0,
       in fact the back is coming from the previous ma
 21 -> the mobile --------------------------------------  2 to 1
 200 -> the mobile host was in MIP mode and after a movement, it 
        swapped for UHMMA2, so it sent a registration packet with
	cn list and locked normal registration, we unlock here.
 201 -> the mobile host was in UHMMA1 mode and after a movement, it 
        swapped for UHMMA2, so it sent a registration packet with
	cn list and locked normal registration, we unlock here.


UHMMA Mode 1 :
If the packet is coming from the MA, check the sequence number,
if it matches stop the retransmission timer (if exist) 
and start old position if not at home.

called by : recv_bnd_ack
call :
*/
char uhmma_back_check(ip, seq, cp, len,status)
struct ip6_hdr *ip;	/* the ip header */
u_short seq;		/* the seq# read in the packet */
u_char *cp;		/* pointer on the end of the back pkt */
int len;		/* length of the back packet */
u_char status;		/* the status read in the packet */
{
    /*----------------------------------
         UHMMA mode 2 
    ----------------------------------*/
    if(global.mobility_mode == UHMMA_MODE2)
    {
	/*------------------------------
	 back is coming from the MA 
	-------------------------------*/
        if(SAME_ADDR6(ip->ip6_src,global.MA_address))
        {
    	    if (seq < global.seq_num)
	    {
	        if(verboseU)
			logtrace("\tUHMMA ; receive back from the MA but the seq# is to small");
		return 1;
	    }
	    if(
		(status != OPT6_BNDA_ACCEPT)&&
		(status != OPT6_BNDA_CN_LIST_KO)&&
		(status != OPT6_BNDA_CN_LIST_OK)
	      )
	    {/* todo ; sort OPT6 by nbBACK value */
	        if(verboseU)
			logtrace("\tUHMMA ; receive back from the MA but the status is bad");
		    return 1;
	    }
            {
                switch (global.nbBACK)
                {
                    case -1:
			goto uhmma2;
                    case 3:
                        global.nbBACK = 1; /* still waiting for the HA */
			if(verboseU)
			    logtrace("\tUHMMA ; receive back from the MA seq#%d, still waiting for the HA new nbBACK %d",seq,global.nbBACK);
			goto seq_check;
                    case 2:
			if(verboseU)
			    logtrace("\tUHMMA ; receive back from the MA seq#%d, all BACK received",seq);
			goto uhmma2;
		    case 20:
			/* the back is coming from the previous ma
			   we have moved and swap from 2 to 0,
			   we do not have changed MA_address, so it still contains
			   MA_old */
			if(verboseU)
			    logtrace("\tUHMMA ; receive back from the previous MA seq#%d, smooth end ",seq);
			goto swap2to0;
		    case 21:
		    {
			struct corr * cop;
			/* the backis coming from the previous ma
			   we have moved and swap from 2 to 1,
			   we do not have changed MA_address, so it still contains
			   MA_old */
			/* MA_address = MA_old
			   MA_old = new care-of
			 */
			store_cn_list(ip,cp,len);
			global.mobility_mode = UHMMA_MODE1;
			global.swap_in_progress = 0;
			if(verboseU)
    			    logtrace("\tUHMMA ; smooth handoff retransmission timer killed %x, packet freed",
				    global.reemis_timer);
			/* destroy the reemission timer */
			DESTROY_TIMER(global.reemis_timer);
			/* free the reemit packet */
			free(global.reemit_reg_packet);
			/* uhmma stuff */
			COPY_ADDR6(global.MA_address,global.MA_old);
			COPY_ADDR6(global.MA_readed, global.MA_address);
			uhmma_move(&ip->ip6_dst);
			/* send BU */			
			update_cn_list();
			FORALLCOR(cop)
			    mobile_move(cop,&ip->ip6_dst);
			goto seq_check;
		    }
		    case 200:
		    case 201:
			if(verboseU)
			    logtrace("\tUHMMA ; receive back from the MA seq#%d",seq);
			global.nbBACK = 100;
			return 1;
                    default:
			if(verboseU)
			    logtrace("\tUHMMA ; receive back from the MA seq#%d but something goes wrong with nbBACK",seq);
            		return 1;
                }
            }
        }
        else
        {
	    /* ------------------------------
	       the back is coming from the HA 
	    --------------------------------*/
            if(SAME_ADDR6(ip->ip6_src, global.HA->addr))
            {
        	if (seq < global.seq_num)
		{
		    if(verboseU)
			logtrace("\tUHMMA ; receive back from the HA but the seq# is to small");
		    return 1;
		}
		if(status != OPT6_BNDA_ACCEPT)
		{
		    if(verboseU)
			logtrace("\tUHMMA ; receive back from the HA but the status is bad");
		    return 1;
		}
                switch (global.nbBACK)
                {
                    case 3:
                        global.nbBACK = 2; /* still waiting for the MA */
			if(verboseU)
			    logtrace("\tUHMMA ; receive back from the HA seq#%d, still waiting for the MA new nbBACK %d",seq,global.nbBACK);
			return 1;
                    case 1:
			if(verboseU)
			    logtrace("\tUHMMA ; receive back from the HA seq#%d, all BACK received",seq);
			goto uhmma2;
		    case 200:
		    case 201:
		    case 100:
		    {
			/*  we include tree cases here : 
			    if nbBACK == 200 or 201, the ha's ack arrives before
			    the ma's one. we assume that every thing is ok,
			    maybe the ma's back is lost, BUT WE KNOW THAT THE
			    MH IS REGISTERED (because we received the ha's ack)
			 */
			struct corr *cop;
			if(verboseU)
			    logtrace("\tUHMMA ; receive back from the HA seq#%d, all BACK received",seq);
			global.swap_in_progress =0;
			/* remove all ch except the ha */
			FORALLCOR(cop)
			    if((cop->type & CORR_USEBND) && 
				!(cop->type & CORR_SETHA))
				del_corr(cop);
			goto uhmma2;
		    }
                    default :
			if(verboseU)
			    logtrace("\tUHMMA ; receive back from the HA seq#%d but something goes wrong with nbBACK",seq);
            		return 1;
                }
            }
            else
		if(verboseU)
		    logtrace("\tUHMMA ; receive back from a CH");
            return 0;
        }
    }
    /* --------------------------------
	UHMMA mode 1 
    ---------------------------------*/
    else if (global.mobility_mode == UHMMA_MODE1)
    {
	/* the packet is coming from the MA */
        if(SAME_ADDR6(ip->ip6_src,global.MA_address)) 
        {
	    /* check the sequence number */
            if (seq >= global.seq_num)
            {
		if(
		    (status != OPT6_BNDA_ACCEPT)&&
		    (status != OPT6_BNDA_CN_LIST_KO)&&
		    (status != OPT6_BNDA_CN_LIST_OK)
	          )
		{
	    	    if(verboseU)
			logtrace("\tUHMMA ; receive back from the MA but the status is bad");
		    return 1;
		}
		if(verboseU)
		    logtrace("\tUHMMA ; receive back from the MA seq#%d, all BACK received",seq);
		goto uhmma2;
	    }
	    else
	    {
		if(verboseU)
		    logtrace("\tUHMMA ; receive back from the MA but the seq# is to small");
                return 1;
	    }
	}
    }
    return 0; /* this is a MIPv6 back */
/*---------------------------
    common stuff for uhmma
---------------------------*/
uhmma2:
    if(verboseU)
        logtrace("\tUHMMA ; retransmission timer killed %x, packet freed",
				global.reemis_timer);
    /* destroy the reemission timer */
    DESTROY_TIMER(global.reemis_timer);
    /* backup the MA address */
    COPY_ADDR6(global.MA_address, global.MA_old);
    /* free the reemit packet */
    free(global.reemit_reg_packet);
    /* timer for sending old position packet */
    if((! SAME_ADDR6(global.HA->home,global.HA->careof))&&
		(global.mobility_mode != MIP_MODE)
	   )
    {
	    global.uMIP_timer=new_timeout(temp.position, 0, \
                        send_old_position, NULL);
    	    if(verboseU)
            	logtrace("\tUHMMA ; old position started %x",global.uMIP_timer);
    }
    global.nbBACK = 0;
/*-----------------------------------------------------------
    check for a synchronisation of the seq number by the MA
------------------------------------------------------------*/
seq_check:
    if(seq>global.seq_num)
    {
	if (verboseU)
	    logtrace("\tUHMMA ; synchronize seq#%d -> %d",global.seq_num,seq);
	global.seq_num = seq;
	
    }
    return 1;


/*-----------------------------------------------------------
    swap from mode 2 to 0 process
------------------------------------------------------------*/
swap2to0:
{
	struct corr * cop;
	struct prefix * vp;

	store_cn_list(ip,cp,len);
	global.mobility_mode = MIP_MODE;
	global.swap_in_progress =0;
	/* useful if we are coming from stop_UHMMA because
	    HA->careof already contains the new careof
	    -> uhmma_minor_reg
	 */
	CLR_ADDR6(global.HA->careof);
	/* send BU */
	update_cn_list();
	FORALLCOR(cop)
	    mobile_move(cop,&ip->ip6_dst);
	global.seq_num =0;
	CLR_ADDR6(global.MA_address);
	CLR_ADDR6(global.MA_old);
	CLR_ADDR6(global.VCOA);
	if(verboseU)
	    logtrace("\tUHMMA ; smooth handoff retransmission timer killed %x, packet freed",
		    global.reemis_timer);
	/* destroy the reemission timer */
	DESTROY_TIMER(global.reemis_timer);
	/* free the reemit packet */
	free(global.reemit_reg_packet);
	DESTROY_TIMER(global.uMIP_timer)
	DESTROY_TIMER(global.back_timer)
	/* depreciate the vcoa prefix */
	if(verboseU)
		logtrace("\tUHMMA ; depreciate vcoa");
	FORALLVP(vp)
	{
	    deprec_pref(vp,temp.forward,temp.forward-5);
	    TAILQ_REMOVE(&global.Vpref, vp, prf_list);
	}
	return;
}


} //end of function

#endif
