#ifdef UHMMA

#include "ndpd.h"
#include "../ndpd-mobility/mobility.h"
#include "../ndpd-uhmma/uhmma.h"
void build_list(int ,char * );

/*
the mobile host was running in UHMMA2.
its moved in a new domain without UHMMA2, so it started a smooth handoff with
the previous MA. the response of the ma contains the cn list of the mobile.
now we have to stoe this list.

called by : uhmma_back_check
call : add_corr
*/
void store_cn_list(ip,cp, len)
struct ip6_hdr *ip;	/* the ip header */
u_char *cp;		/* cn list */
int len;		/* option length */
{
    struct in6_addr * home;
    u_char * par;
    struct corr4list tmp;
    int ii;
    int nbCN;
    char type;
    u_char ln;
    
    GET8(type,cp);
    GET8(ln,cp);
    
    if ((type != SUB_OPT6_CN) 
        ||(ln <= 0))
    {
	if(verboseU)
	    logtrace("\tUHMMA ; received a back with a buggy cn list");
	return;
    }
    nbCN = (ln 		/* the sub option length without the type n length fields*/
		-6 	/* pad */
		-16)	/* home address */
		/sizeof(struct corr4list);
    cp+=6;	/* pad */
    GETADDR(home, cp);
    par = cp;
    for(ii=0;ii<nbCN;ii++)
    {
	struct corr * cop;
	bcopy(par,&tmp,sizeof(struct corr4list));
	/* add the CH structure */
	/* the HA */
	if(SAME_ADDR6(global.HA->addr,tmp.addr))
	{
	    global.HA->seq = tmp.seq;
/*	    COPY_ADDR6(ip->ip6_dst,global.HA->careof);*/
	    global.HA->flags &=~0x20;	/* remove the C flag */
	    global.HA->flags |= OPT6_BNDU_ACK;	/* add the A flag because
	    the ma probably removed it after the first home registration
	    sending */
	}
	/* a new CN */
	else
	{
	    if(Calloc(cop,corr) == NULL)
	    {
		if(verboseU)
		    logtrace("\tUHMMA ; problem during cn list registration");
		return;
	    }
	    COPY_ADDR6(tmp.addr,cop->addr);
	    cop->type=tmp.type;
	    cop->flags=tmp.flags &~0x20;	/* remove the C flag */
	    cop->type = CORR_SETHA | CORR_USEBND;
	    cop->plen=tmp.plen;
	    cop->lifetime=tmp.lifetime;
	    cop->seq=tmp.seq;

	    COPY_ADDR6(global.HA->home,cop->home);
/*	    COPY_ADDR6(ip->ip6_dst,cop->careof);*/

	    add_corr(cop);
	}

	/* next one */
	par+=sizeof(struct corr4list);
    }
}


/*
the mobile host was on a mip/uhmma1 domain and moved into a uhmma2 domain
it has to send it cn list to the ma

called by : swap_mode
call : build_list, send_kernel
*/
void send_cn_list(MA)
struct in6_addr * MA;		/* the MA destination address */
{
	int i,d, cc, olen;
	struct corr * cop;
	int cnnb = 0;
	char * list;
	int buffer_size;
	int byte_8_length;
	int sub_option_length ;
	int option_length;
	
	if(verboseU)
	    logtrace("\tUHMMA ; will send cn list");
	    
	/* find the number of cn */
	FORALLCOR(cop)
	{
	    if(cop->type & CORR_USEBND)
		cnnb++;
	}
	list = (char *)malloc (cnnb * sizeof(struct corr4list));
	if(list == NULL) 
	{
	    if(verboseU)
		logtrace("\tUHMMA ; not enough memory to send corr list");
	    return;
	}
	build_list(cnnb,list);
	sub_option_length = cnnb * sizeof(struct corr4list)+6+16;
			/* don't forget the two first bytes of the sub option (pad for alignment)
			the length and the size must not be included
			don't forget the home address*/
	option_length = 11+sub_option_length+2;\
			/* size of the BACK option , include the sub-option
			don't forget the first line of the sub option
			and the home address*/
	buffer_size = option_length+5;
			/* size of the option + the type + the length field
			    and don't forget the alignment
			*/
	byte_8_length = buffer_size/8 - 1;
			/* 8byte size ; a corr4list structure is 32byte long
			don't forget the first line of the sub option
			and the home address*/
	/* fill a destination option header with the binding ack */
    {
	char babuf[buffer_size];
	register u_char *cp;
	
	cp = (u_char *)babuf;

	SET8(IP6_NHDR_NONH, cp);
	SET8(byte_8_length, cp);   
	SET8(OPT6_PAD_0, cp);
	SET8(OPT6_BINDING_ACK, cp);
	SET8(option_length, cp);
	SET8(OPT6_BNDA_CN_LIST_OK, cp);
	SET16(global.seq_num, cp);
	SET32(temp.lifetime, cp);
	SET32(temp.refresh, cp);
	/* sub option with cn list */
	SET8(SUB_OPT6_CN,cp);
	SET8(sub_option_length , cp);

	SET16(0,cp);
	SET32(0,cp);
	SETADDR(global.HA->home, cp);
	bcopy(list,cp,cnnb*32);

	if(send_kernel("send cn list",babuf,buffer_size,
			&global.HA->careof,MA,0,0)==1)
	    if(verboseU)
		logtrace("\tUHMMA ; send cn list to %s seq#%d",
			inet6pr(MA),global.seq_num);

	global.reemit_reg_packet = malloc (sizeof(babuf));
	/* start the reemit timer */
	if (global.reemit_reg_packet ==NULL)
	{
	    logerr("not enough memory to store reg packet ; retransmit canceled");
	}    
	else
	{
	    global.pkt_size = buffer_size;
	    memcpy(global.reemit_reg_packet, babuf, sizeof(babuf));
	    /* timer for re emission */
	    global.reemis_timer=new_timeout(temp.retransmission, 0, (func_to) \
		    resend_cn_list, global.reemit_reg_packet);
	    if(verboseU)
		logtrace("\tUHMMA ; reemit timer started %x",global.reemis_timer);
	}
    }
	/* free the list */
	free(list);

}
/*
build the correspondents' list to send in the BACK packet
(it is a list of corr4list structures) 
by using the correspondents' list of the mobile host
(it is a list of corr structures) 

note : do not include virtual correspondent (prefix)
called by : send_cn_list
call :
*/
void build_list(nb,list)
int nb;			/* number of CH */
char * list;		/* the list to fill */
{
	struct corr4list tmp;
	struct corr * cop;
	char * l = list;
	
	FORALLCOR(cop)
	    if(cop->type & CORR_USEBND)
	    {
	    COPY_ADDR6(cop->addr,tmp.addr);
	    tmp.type = cop->type;
	    tmp.flags = cop->flags;
	    tmp.plen = cop->plen;
	    tmp.seq = cop->seq;
	    tmp.lifetime = cop->lifetime;
/*	    tmp.retcnt = cop->retcnt;*/
	    bcopy(&tmp,l,sizeof(struct corr4list));
	    l+=sizeof(struct corr4list);
	    }
}

/*
the reemission timer for a cn list has expired 
so we have to resend the Ureg AND the cn list

called by : run_timeout
call :
*/
void resend_cn_list(pkt)
char * pkt;	/* the packet with the cn list */
{
	char * upd;

	if (verboseU) 
		logtrace("\tUHMMA ; will REsend reg");
	
	global.mobility_mode = UHMMA_MODE2_FROM_MIP;
	build_send_reg(&global.HA->careof,temp.lifetime,0,1);
	global.mobility_mode = UHMMA_MODE2;
		
	/* send */
	if(send_kernel("resend uhmma reg",pkt,global.pkt_size,&global.HA->careof,
			&global.MA_address,0,0)==0)
	    return;
	if (verboseU) 
		logtrace("\tUHMMA ; re send reg to %s",
				 inet6pr(&global.MA_address));
	/* we will reuse the old timer */
	set_timeout(global.reemis_timer,temp.retransmission, 0, (func_to) \
		    resend_cn_list, global.reemit_reg_packet);
	logtrace("\tUHMMA ; reemit timer re started %x",global.reemis_timer);
    
}


/*
update the cn list after a movement or a swap 
(ie look if the cn and the mh are in the same domain.

note ; the home agent always need the vcoa

if the mobile uses paging, the VCOA is alway used.

called by : send_uhmma_minor_reg, check_uhmma_back, swap_mode, stop_smooth
call :
*/
void update_cn_list()
{
    struct corr * cop;
    
    FORALLCOR(cop)
	if(global.mobility_mode == MIP_MODE)
	    CLR_ADDR6(cop->VCOA);
	else
	    if(
			(same_prefix(&cop->addr,&global.VCOA,global.site_len)==0)
			|| (cop->flags & OPT6_BNDU_HR)
	      )
		COPY_ADDR6(global.VCOA,cop->VCOA);
	    else
		CLR_ADDR6(cop->VCOA);
}

#endif
