/* Manage tunnel end at the mobile */
/* 
this functions is initially written for a mobile in MIPv6.
we copy them to hack them. 
we use tug1, tug0 is the tunnel to the ha.
we don't need bpf stuff for correspondent nodes detection.
 */

#ifdef UHMMA
#include "ndpd.h"
#include "../ndpd-mobility/mobility.h"
#include "uhmma.h"

#include <netinet/if_tug6.h>

#define CAPLEN		72
#define TUGCSIZE	8


/*
Add a tunnel to the MA, we use tug1, tug0 is the tunnel to the ha.

called by : send_uhmma_major_reg, send_uhmma_minor_reg
call :
*/
void
madd_ma_tunnel(agp)
struct corr *agp;	/* the MA */
{
	struct if_laddrreq ifr6;
	struct ifreq ifr;
	struct sockaddr_in6 *sin;
	char dname[12];
	int d, unit = 0;
	u_int val;
	char * name = "tug1";

	if (verboseU)
		logtrace("\tUHMMA ; will set up tunnel end on %s from %s",
			 name, inet6pr(&agp->addr));

	/* UDP/IPv6 socket for interface ioctls */
	if ((d = socket(PF_INET6, SOCK_DGRAM, 0)) < 0) {
		logperror("socket (mobility-agent/add)");
		return;
	}

	/* just set the end point of tunnel, making it RUNNING... */

	bzero(&ifr6, sizeof(ifr6));
	strncpy(ifr6.iflr_name, name, IFNAMSIZ);
	sin = (struct sockaddr_in6 *)&ifr6.addr;
	sin->sin6_len = sizeof(struct sockaddr_in6);
	sin->sin6_family = AF_INET6;
	COPY_ADDR6(agp->addr, sin->sin6_addr);

	if (ioctl(d, SIOCSTUGLADDR, &ifr6) < 0) {
		logperror("ioctl (SIOCSTUGLADDR)");
		(void) close(d);
		return;
	}

	bzero(&ifr, sizeof(ifr));
	strncpy(ifr.ifr_name, name, IFNAMSIZ);
	
	if (ioctl(d, SIOCGIFFLAGS, &ifr) < 0) {
		logperror("ioctl (SIOCGIFFLAGS)");
		(void) close(d);
		return;
	}

	ifr.ifr_flags |= IFF_UP;

	if (ioctl(d, SIOCSIFFLAGS, &ifr) < 0) {
		logperror("ioctl (SIOCSIFFLAGS)");
		(void) close(d);
		return;
	}

}

/*
Delete a tunnel 
FD has not write this code yet ....

called by : send_uhmma_minor_reg, close_back_tug
call :
*/

void
mdel_ma_tunnel(agp)
struct corr *agp;	/* the MA */
{
	/* TODO ??? down and unconfig */

	if (verboseU)
		logtrace("will release tunnel end on %s from %s",
			 "tug1", inet6pr(&agp->addr));

}

#endif
