/* a mobile need to swap from a mobility mode to an other */

#ifdef UHMMA

#include "ndpd.h"
#include "../ndpd-mobility/mobility.h"
#include "../ndpd-uhmma/uhmma.h"



/*
does the stuff for a mobility mode swap.

M  -> U1 : just change the mobility mode
M  -> U2 : send a registration to the local ma
	   send a cn list in the registration packet and change the mode
	   do vcoa stuff (create, add, save)
U1 -> U2 : idem
U1 -> M  : change the mobility mode and send a forward packet to the previous ma
U2 -> U1 : start a smooth handoff with the previous ma and lock the sending
	   of registration and BU. The mode will be changed and the registration/
	   BU stuff will be done when the back will be here.
U2 -> M  : start a smooth handoff with the previous ma and lock the sending
	   of registration and BU. The mode will be changed and the registration/
	   BU stuff will be done when the back will be here.

called by : major_move, minor_move, stop_UHMMA
call : send_smooth, build_send_reg, send_uhmma_major_reg, update_cn_list, send_cn_list
*/
void swap_mode(careof)
struct in6_addr * careof;	/* the new position of the mobile host */
{
    struct prefix * vp;
    struct corr * cop;
    
    if (global.need_to_swap == -1) return;

    switch(global.mobility_mode)
    {
	case MIP_MODE:
	    if(verboseU)
		logtrace("\tUHMMA ; swap from MIP");
	    switch(global.need_to_swap)
	    {
		case UHMMA_MODE1:
		    if(verboseU)
			logtrace("\tUHMMA ; to Mode 1");
		    global.mobility_mode = UHMMA_MODE1;
		    COPY_ADDR6(global.MA_readed,global.MA_address);
		    send_uhmma_major_reg(careof);
		    update_cn_list();
		    FORALLCOR(cop)
			mobile_move(cop,careof);
		    return;    
		case UHMMA_MODE2:
		{
		    struct in6_addr vcoa;
		    struct interface *itf;
		    
		    if(verboseU)
			logtrace("\tUHMMA ; to Mode 2");
		    global.swap_in_progress = 1;
		    global.nbBACK = 200;
		    /* send Ureg with special flag */
		    COPY_ADDR6(global.MA_readed,global.MA_address);
		    COPY_ADDR6(*careof,global.HA->careof);
		    global.mobility_mode = UHMMA_MODE2_FROM_MIP;
		    global.seq_num ++;
		    build_send_reg(careof,temp.lifetime,0,1);
		    global.mobility_mode = UHMMA_MODE2;
			/* remove old vcoas */
			FORALLVP(vp)
			{
	    		deprec_pref(vp,temp.forward,temp.forward-5);
	    		TAILQ_REMOVE(&global.Vpref, vp, prf_list);
			}
		    /* build the vcoa */
		    vcoa = make_VCOA(careof);
		    /* add vcoa */
		    FORALLIFP(itf)
			if(IS_UP(itf))
			    add_VCOA(itf, &vcoa);
		    /* store the vcoa */
		    COPY_ADDR6(vcoa,global.VCOA);
		    COPY_ADDR6(vcoa,global.HA->VCOA);
		    /* send back + cn list */
		    send_cn_list(&global.MA_address);
		    return;
		}
	    }
	case UHMMA_MODE2 :
	    if(verboseU)
		logtrace("\tUHMMA ; swap from Mode 2");
	    switch(global.need_to_swap)
	    {
		case UHMMA_MODE1:
		    if(verboseU)
			logtrace("\tUHMMA ; to Mode 1");
		    send_smooth(careof);
		    global.swap_in_progress = 1;
		    /* do not set the new mobility mode yet,
		       we have to wait for the cn list */
		    /* swap in progress is used to lock the sending of UHMMA
		       registration to the local MA and BU to CN.*/
		    global.nbBACK = 21;
		    COPY_ADDR6(global.MA_old,global.MA_address);
		    /* to bypass filters in recv_bnd_ack we have to
		       copy the address of the previous MA in the local
		       MA, the local MA is not saved because in 
		       send_uhmma_reg, we will use MA_readed to initialize
		       MA_address.
		     */
		    COPY_ADDR6(*careof,global.MA_old);
		    // we use MA_old to store the careof :-(
		    return;
		case MIP_MODE:
		    if(verboseU)
			logtrace("\tUHMMA ; to MIP");
		    send_smooth(careof);
		    global.swap_in_progress = 1;
		    global.nbBACK = 20;
		    COPY_ADDR6(global.MA_old,global.MA_address);
		    COPY_ADDR6(*careof,global.MA_old);
		    return;
	    }
	case UHMMA_MODE1 :
	    if(verboseU)
		logtrace("\tUHMMA ; swap from Mode 1");
	    switch(global.need_to_swap)
	    {
		case UHMMA_MODE2:
		{
		    struct in6_addr vcoa;
		    struct interface *itf;
		    
		    if(verboseU)
			logtrace("\tUHMMA ; to Mode2");
		    global.swap_in_progress = 1;
		    global.nbBACK = 200;
		    /* send Ureg with special flag */
		    COPY_ADDR6(global.MA_readed,global.MA_address);
		    COPY_ADDR6(*careof,global.HA->careof);
		    global.mobility_mode = UHMMA_MODE2_FROM_MIP;
		    global.seq_num ++;
		    build_send_reg(careof,temp.lifetime,0,1);
		    global.mobility_mode = UHMMA_MODE2;
			/* remove old vcoas */
			FORALLVP(vp)
			{
	    		deprec_pref(vp,temp.forward,temp.forward-5);
	    		TAILQ_REMOVE(&global.Vpref, vp, prf_list);
			}
		    /* build the vcoa */
		    vcoa = make_VCOA(careof);
		    /* add vcoa */
		    FORALLIFP(itf)
			if(IS_UP(itf))
			    add_VCOA(itf, &vcoa);
		    /* store the vcoa */
		    COPY_ADDR6(vcoa,global.VCOA);
		    COPY_ADDR6(vcoa,global.HA->VCOA);
		    /* send back + cn list */
		    send_cn_list(&global.MA_address);
		    return;
		}
		case MIP_MODE:
		    if(verboseU)
			logtrace("\tUHMMA ; to MIP");
		    send_forward(careof);
		    CLR_ADDR6(global.MA_old);
		    CLR_ADDR6(global.MA_address);
		    CLR_ADDR6(global.VCOA);
		    DESTROY_TIMER(global.reemis_timer)
		    DESTROY_TIMER(global.uMIP_timer)
		    DESTROY_TIMER(global.back_timer)
		    /* depreciate the vcoa prefix */
		    if(verboseU)
	    		    logtrace("\tUHMMA ; depreciate vcoa");
		    FORALLVP(vp)
		    {
			deprec_pref(vp,temp.forward,temp.forward-5);
			TAILQ_REMOVE(&global.Vpref, vp, prf_list);
		    }

		    global.mobility_mode = MIP_MODE;
		    update_cn_list();
		    return;
	    }
    }
    global.need_to_swap  = -1;  
}

#endif
