/* Correspondent management for a mobile node */

#ifdef UHMMA

#include "ndpd.h"
#include "../ndpd-mobility/mobility.h"

#include "../ndpd-uhmma/uhmma.h"
/*
check the kind of movement, major_move or minor_move
if minor_move, check if micro mobility or macro mobility

called by : uhmma_back_check
call : send_uhmma_major_reg, send_uhmma_minor_reg
*/
void uhmma_move(newca)
struct in6_addr *newca;		/* new position of the mobile host */
{
	if (verboseU)
		logtrace("\tUHMMA move from %s to %s",
			 inet6pr(&global.HA->careof),
			 inet6pr(newca));

	/*
	 * Return to home:
	 */
	if (IN6_ARE_ADDR_EQUAL(newca, &global.HA->home)) 
	{
	    send_uhmma_major_reg(newca);
	    return;
	}

	/*
	 * Move from home:
	 */
	if (IS_ANYADDR6(global.MA_old)) 
	{
	    send_uhmma_major_reg(newca);
	    return;
	}

	/*
	 * Move from a place to another one:
	 */
	send_uhmma_minor_reg(newca);
}
/*
same as uhmma_move excepted that it check if the mobile need to swap
before perform the movement.

return 1 if mobile_chgif needs to execute FORALLCOR mobile_move

called by : mobile_chgif
call : send_uhmma_major_reg, send_uhmma_minor_reg, same _prefix
*/
char uhmma_move_mode(newca)
struct in6_addr *newca;		/* new position of the mobile host */
{
	if (verboseU)
		logtrace("\tUHMMA move from %s to %s",
			 inet6pr(&global.HA->careof),
			 inet6pr(newca));
	/***************************
	 *     Return to home:
	 ***************************/
	if (IN6_ARE_ADDR_EQUAL(newca, &global.HA->home)) 
	{
	    send_uhmma_major_reg(newca);
	    return 1;
	}

	/*****************************************************
	 *     Move from home or from  a mip domain :
	 *****************************************************/
	
	if (IS_ANYADDR6(global.MA_old)) 
	{
	    /* no MA here */
    	    if(IS_ANYADDR6(global.MA_readed))
	    {
		/* swap from 2 | 1 to 0 */
		struct corr *cop;
	        if(global.mobility_mode > MIP_MODE)
		{
		    global.mobility_mode = MIP_MODE;
		    if(verboseU)
			logtrace("\tUHMMA ; swap to MIP because no MA availabled");
		}
		return 1;
	    }
	    /* we find a MA */
	    /* mobile host in mode 2 */
    	    if (global.mobility_mode == UHMMA_MODE2)
	    {
		/* ma in mode 1 */
		if(global.MA_mode == UHMMA_MODE1)
		{
		    /* swap from 2 to 1 */
		    global.mobility_mode = UHMMA_MODE1;
		    if(verboseU)
			logtrace("\tUHMMA ; swap to MODE1 because MA only runs this mode");
		    send_uhmma_major_reg(newca);
		    return 1;
		}
		/* ma in mode 2 */
		else
		{
		    send_uhmma_major_reg(newca);
		    return 0;
		}
	    }
	    /* mobile host in mode 1 */
    	    if (global.mobility_mode == UHMMA_MODE1)
	    {
		send_uhmma_major_reg(newca);
		return 1;
	    }
	    /* mode mip */
	    else
		if(global.mobility_mode < global.mobility_mode_init)
		{
		    global.need_to_swap =(global.mobility_mode_init > global.MA_mode ?
				    global.MA_mode : global.mobility_mode_init );
		    swap_mode(newca);
		    return 0;
		}
		else
		{
		    return 1;
		}
	}

	/*****************************************************
	 *        Move from a place to another one:
	 *****************************************************/

	/* macro move */
	if(! same_prefix(&global.MA_readed,&global.MA_old,global.site_len))
	{
	    /* ma mode is greater than mobile host mode */
	    if(global.mobility_mode < global.MA_mode)
		/* the initial mobile host mode is greater than the actual mode */
		if(global.mobility_mode < global.mobility_mode_init)
		{
		    global.need_to_swap =(global.mobility_mode_init > global.MA_mode ?
				    global.MA_mode : global.mobility_mode_init );
		    swap_mode(newca);
		    return 0;
		}
	}
	/* macro and micro moves */
	send_uhmma_minor_reg(newca);
	return 1;
}


#endif
