/* mobile registration
    UHMMA MODE
*/

#ifdef UHMMA

#include "ndpd.h"
#include "../ndpd-mobility/mobility.h"
#include "../ndpd-uhmma/uhmma.h"

/*
close the tug with the old MA.
the timeout is freed when it expires by run_timeout because refcnt == 0

called by : run_timeout (because the timeout expires)
call : mdel_ma_tunnel
*/
void clear_back_tug ()
{
    struct corr MA;
    
    if (verboseU)	
        logtrace("\tUHMMA ; back timer killed %x",global.back_timer);
    /* cleanup some variables */
    DESTROY_TIMER(global.back_timer);
    COPY_ADDR6(global.MA_old,MA.addr);
    CLR_ADDR6(global.MA_old);
    CLR_ADDR6(global.MA_address);
    CLR_ADDR6(global.VCOA);
    /* check if the last uhmma registration is still here */
    if (global.reemis_timer) 
    {
		DESTROY_TIMER(global.reemis_timer);
		free(global.reemit_reg_packet);
		global.reemit_reg_packet=NULL;
    }
}

/*
send the packet pkt to the kernel.
this packet is a registration packet. the MA does not send a BACK. so 
the registration have to be resent.

don't forget to remove the vcoa with is the gw (so the dest addr)
before re sending the packet
don't forget to re add this vcoa after sending

called by : run_timeout (because reemis_timer expires)
call : set_timeout, send_kernel
*/
void resend_uhmma_reg(pkt)
char * pkt;	/* the buffer to send */
{
	char * upd;
	char result;

	/* check retransmission */
	if (global.reg_ret++ >= global.reg_ret_max)
	{
		if(verboseU)
			logtrace("\tUHMMA ; too munch retransmissions");
		/* swap to mobile IP */
		stop_UHMMA(&global.HA->careof);
		return;
	}
    /* increment seq# */
    upd = pkt+6;
    global.seq_num++;
    SET16(global.seq_num, upd);
    
	/* send */
	result =
	send_kernel("resend uhmma reg",pkt,global.pkt_size,
			&global.HA->careof,
			&global.MA_address,0,0);
	if(result == 0)
	    return;
	if (verboseU) 
		logtrace("\tUHMMA ; re send reg to %s seq#%d ret#%d",
				 inet6pr(&global.MA_address),
				 global.seq_num,
				 global.reg_ret);
	/* we will reuse the old timer */
	set_timeout(global.reemis_timer,temp.retransmission, 0, (func_to) \
		    resend_uhmma_reg, global.reemit_reg_packet);
	logtrace("\tUHMMA ; reemit timer re started %x",global.reemis_timer);
    
	/* todo :
	    compare MA address and the packet destination
	*/

}

/*
build a registration packet.
this packet is composed by : 
- DO BU (with a MA sub-option and a HA sub-option if full == 1)
- DO H@ 
the packet is sent to the kernel.
the packet is back uped in memory (reemit_reg_packet) if retrans==1.
reemis_timer is turned on. if it expires, the packet ll be resent. 
BACK(s) stop(s) this timer.

we know that the vcoa is not on itf !
this is important because if we are on a cip network the vcoa is the gw 
so the dest address !!!

IMPORTANT : full must be 1 or 0 !!!

called by : send_uhmma_major_reg, send_uhmma_minor_reg, wake_up
	    mobile_uhmma_clean, swap_mode, send_route, resend_cn_list
call : 
*/
void build_send_reg(newcoa,life,retrans,full)
struct in6_addr * newcoa;	/* packet source */
int life;			/* lifetime for this registration */
char retrans;			/* shall restransmit ? */
char full;			/* include HA and MA in this registration ? */
{
	register u_char *cp;
	int byte_length = (full==1) ? 56:32;
	int byte_8_length = (byte_length -8) / 8;
	char bubuf[byte_length];

	// build the packet
	cp = (u_char *)bubuf;

	SET8(IP6_NHDR_NONH, cp);
	SET8(byte_8_length, cp);
	/* DO BU  */
	SET8(OPT6_ROUTE_UPD, cp);
/*	SET8(OPT6_BINDING_UPD, cp);*/
	SET8(full == 1 ? 32:8, cp);
	SET8(global.mobility_mode, cp);
	SET8(0, cp); /* pref length, we do not put 64 or the virtual prefix
	 length here because there is no need for the MA to be proxy for llocal
	    address and multicast addresses */
	SET16(global.seq_num, cp);
	SET32(life, cp);	/* lifetime */
    if(full == 1)
    {
		/* sub option MA */
		SET8(SUB_OPT6_MA, cp);
		SET8(22, cp);
		SET8(OPT6_PAD_0, cp);
		SET8(OPT6_PAD_0, cp);
		SETADDR(global.MA_old, cp);
		SET32(0,cp);
        /* sub option HA 
        SET8(SUB_OPT6_HA, cp);
        SET8(UHMMA_HA_SUBO_LENGTH, cp);
        SET8(OPT6_PAD_0, cp);
        SET8(OPT6_PAD_0, cp);
        SETADDR(global.HA->addr, cp);*/
    }	
	SET8(OPT6_PAD_0, cp);
	SET8(OPT6_PAD_0, cp);
	/* DO H@ */
	SET8(OPT6_HOME_ADDR, cp);
	SET8(16, cp);       /* length */
	SETADDR(global.HA->home, cp);
	/* send */
	{
    	struct sockaddr_in6 sin;
		int d, cc;

		/* get a fresh socket */
		if ((d = socket(PF_INET6, SOCK_RAW, IPPROTO_DSTOPTS)) < 0) 
		{
			logtrace("\tUHMMA ; socket UREG");
			return;
		}
		(void) shutdown(d, 0);
		/* bind it */
		bzero((caddr_t)&sin, sizeof(sin));
		sin.sin6_family = AF_INET6;
		sin.sin6_len = sizeof(sin);
		COPY_ADDR6(*newcoa, sin.sin6_addr);
		if (bind(d, (struct sockaddr *)&sin, sizeof(sin)) < 0) 
		{
			logtrace("\tUHMMA ; bind UREG");
			(void) close(d);
			return;
		}
		/* connect it */
		COPY_ADDR6(global.MA_address, sin.sin6_addr);
		if (connect(d, (struct sockaddr *)&sin, sizeof(sin)) < 0) 
		{
			logtrace("\tUHMMA ; connect UREG");
			(void) close(d);
			return;
		}
		/* security policy */
		if ((ipsec_len > 0) &&
		    (setsockopt(d, IPPROTO_IPV6, IPV6_IPSEC_POLICY,
				ipsec_policy, ipsec_len) < 0)) 
		{
			logperror("\tUHMMA ; setsockopt(home registration policy)");
			(void) close(d);
			return;
		}
	    disable_homeaddr(d);
		/* send */
		cc = write(d, bubuf, sizeof(bubuf));   
		if (cc != sizeof(bubuf))
		{
			logtrace("\tUHMMA ; write UREG");
		    return;
		}
		(void) close(d);
	}
    
	if (verboseU) 
	    if(
            (full ==1)
	       )
		logtrace("\tUHMMA ; send reg to %s seq#%d full=%d",
				 inet6pr(&global.MA_address),global.seq_num,full);
    	/* backup packet for re emission */
	global.reg_ret= 0;
    if(retrans == 1)
    {
	global.reemit_reg_packet = malloc (sizeof(bubuf));
	/* start the reemit timer */
	if (global.reemit_reg_packet ==NULL)
	{
	    logerr("not enough memory to store reg packet\nretransmit canceled");
	}    
	else
	{
	    global.pkt_size = byte_length;
	    memcpy(global.reemit_reg_packet, bubuf, sizeof(bubuf));
	    /* timer for re emission */
	    global.reemis_timer=new_timeout(temp.retransmission, 0, (func_to) \
		    resend_uhmma_reg, global.reemit_reg_packet);
	    if(verboseU)
		logtrace("\tUHMMA ; reemit timer started %x",global.reemis_timer);
	}
    }
}

/*
does the stuff for a major move :

build and send a registration packet.

if the mobile is coming back to home :
- create a tug (old MA)
- turn on back timer (to close the tug after a lifetime)
- stop uMIP_timer (old position)

if the mobile is going out :
- create a tug (local MA)

stop the uMIP timer witch is used to send periodicaly old position pkt.
it will be restarted when the back will be here.

UHMMA mode 2 :
for a major move the MA ll send a BU to the HA even if the move is an
intra site move, so the mobile has to wait for two BACK, nbBACK = 3

called by : major_move, uhmma_move
call : build_send_reg, madd_ma_tunnel, new_timeout
*/
void send_uhmma_major_reg(newcoa)
struct in6_addr * newcoa;	/* the new position of the mobile host */
{
	/*--------------------------------
	 coming from the home network 
 	---------------------------------*/
	if (IS_ANYADDR6(global.MA_old))
	{
		struct corr tug_dest;
		struct interface * itf;
		struct in6_addr vcoa;
	    
		if (verboseU) 
		    logtrace("\tUHMMA ; will process a major move");
		COPY_ADDR6(global.MA_readed,global.MA_address );
		
		/* hack for ike */
		global.MA->type = CORR_USEBND | CORR_SETHA;
/*		global.MA->plen = global.MA_plen;*/
		global.MA->plen = 128;
		COPY_ADDR6(global.MA_address,global.MA->addr);
		COPY_ADDR6(global.HA->home, global.MA->home);
		COPY_ADDR6(*newcoa, global.MA->careof);
/*		mget_security(global.MA);*/
		add_homeaddr(global.MA);
		/* send registration packet */
		global.seq_num ++;
		build_send_reg(newcoa,temp.lifetime,1,1);
		/* build the vcoa */
		vcoa = make_VCOA(newcoa);
		/* add vcoa */
		FORALLIFP(itf)
		    if(IS_UP(itf))
				add_VCOA(itf, &vcoa);
		/* store the vcoa */
		COPY_ADDR6(vcoa,global.VCOA);
		COPY_ADDR6(vcoa,global.HA->VCOA);
	}
	/*-------------------------------
	 coming back to the home network 
	--------------------------------*/
	else
	{
	    struct prefix *vp;
	    /* no need to open a tug to the old MA ; because this tunnel is allready up : check this theory ;-)
	    */
	    if (verboseU) 
			logtrace("\tUHMMA ; will process for a 'back to home' move");
	    /* send UHMMA reg to the old MA on behalf of the current MA */
	    COPY_ADDR6(global.MA_old, global.MA_address);
	    /* send registration packet */
	    global.seq_num +=1;
	    build_send_reg(newcoa,temp.home,1,1);
	    
	    /* start back to home timer */
   	    global.back_timer=new_timeout(temp.home, 0, (func_to) \
			clear_back_tug, NULL);
	    if (verboseU)	
			logtrace("\tUHMMA ; back timer started %x",global.back_timer);
	    /* stop old position process */
	    DESTROY_TIMER(global.uMIP_timer);
	    /* depreciate the vcoa prefix */
	    if(verboseU)
		    logtrace("\tUHMMA ; depreciate vcoa");
	    FORALLVP(vp)
	    {
			deprec_pref(vp,temp.home,temp.home-5);
			TAILQ_REMOVE(&global.Vpref, vp, prf_list);
	    }
		/* hack for ike */
/*		mdel_security(global.MA);*/
		del_homeaddr(global.MA);
		bzero (global.MA, sizeof(struct corr));
	}
	/*------------------------------
	   common stuff
	-------------------------------*/
	if(global.mobility_mode == UHMMA_MODE2)
	{
	    global.nbBACK =-1;
	    if (verboseU) 
			logtrace("\tUHMMA ; nbBACK %d",global.nbBACK);
	    COPY_ADDR6(*newcoa,global.HA->careof);
	}
}

/*
does the stuff for a minor move for the UHMMA mode only, if the mobile
runs UHMMA mode1 the MIPv6 stuff is done by the caller function.

a minor move is a movement netween two differents networks. 
this two networks are not the home network.
a minor move can be a micro mobility movement or a macro mobility movement.

micro mobility :

Mode 1 & 2 : 

build and send a registration packet.
stop the uMIP timer witch is used to send periodicaly old position pkt.
it will be restarted when the back (MA) will be here.
close the tunnel to the old ma n open a tunnel to the new ma

Mode 2 :
the HA will not be adversised, so the mobile waits fot one BACK; nbBACK = -1


macro mobility :

Mode 1 & 2 :

build and send a registration packet.
stop the uMIP timer witch is used to send periodicaly old position pkt.
it will be restarted when the back (MA) will be here.
close the tunnel to the old ma n open a tunnel to the new ma

Mode 2 :

the mobile has to wait for two back (MA n HA) so nbBACK =3;

called by : minor_move, uhmma_move
call : build_send_reg, madd_tunnel, 
*/
void send_uhmma_minor_reg(newcoa)
struct in6_addr * newcoa;	/* the new position of the mobile */
{
    if (verboseU) 
		logtrace("\tUHMMA ; will perform a minor move");

    /*--------------------
       micro mobility  
    --------------------*/
    /* do not copy ma_readed in ma_address, we have to be managed by the
       same ma as long as we stay in this domain
     */
    if(same_prefix(&global.MA_readed,&global.MA_old,global.site_len))
    {
		/* ack for ike */
		COPY_ADDR6(*newcoa, global.MA->careof);
		del_homeaddr(global.MA);		
		add_homeaddr(global.MA);		
		/* send registration packet */
		global.seq_num++;
		build_send_reg(newcoa,temp.lifetime,1,1);
		/* stop the old position timer, it ll be restarted after reception of the BACK */
		DESTROY_TIMER(global.uMIP_timer);
		if(global.mobility_mode == 2)
		{
		    global.nbBACK =-1;
		    if (verboseU) 
				logtrace("\tUHMMA ; micromobility ; waiting for one back");
		    COPY_ADDR6(*newcoa,global.HA->careof);
		}
    }
    /*--------------------
       macro mobility 
    ---------------------*/
    else
    {
		COPY_ADDR6(global.MA_readed,global.MA_address );
		/* check the mode */
		/* swap from Mode 2 or 1 to MIP */
		if(IS_ANYADDR6(global.MA_readed))
		    if(global.mobility_mode >MIP_MODE)
		    {
				global.need_to_swap=MIP_MODE;
				swap_mode(newcoa);
				return;
		    }
		/* swap from Mode 2 to mode 1 */
		if(global.mobility_mode == UHMMA_MODE2)
		    if(global.MA_mode == UHMMA_MODE1)
		    {
				global.need_to_swap=UHMMA_MODE1;
				swap_mode(newcoa);
				return;
		    }
		{
		/* the VCOA changes */
/*			struct corr MA;*/
			struct prefix * vp;
			struct interface * itf;
			struct in6_addr vcoa;

/*			COPY_ADDR6(global.MA_old,MA.addr);*/
			/* stop the old position timer, 
			it ll be restarted after reception of the BACK */
			DESTROY_TIMER(global.uMIP_timer);
next:
			/* hack for ike */
/*			mdel_security(global.MA);*/
			COPY_ADDR6(global.MA_address,global.MA->addr);
			COPY_ADDR6(*newcoa, global.MA->careof);
			global.MA->plen = global.MA_plen;
			del_homeaddr(global.MA);
			add_homeaddr(global.MA);
/*			mget_security(global.MA);*/
			/* send registration packet */
			global.seq_num++;
			build_send_reg(newcoa,temp.lifetime,1,1);
			/* depreciate the vcoa prefix */
			if(verboseU)
		        logtrace("\tUHMMA ; depreciate vcoa");
			FORALLVP(vp)
			{
			    deprec_pref(vp,temp.forward,temp.forward-5);
			    TAILQ_REMOVE(&global.Vpref, vp, prf_list);
			}
			/* build n add the vcoa to all itf */
			vcoa = make_VCOA(newcoa);
			FORALLIFP(itf)
			    if(IS_UP(itf))
					add_VCOA(itf, &vcoa);
			/* store the vcoa */
			COPY_ADDR6(vcoa,global.VCOA);
	
			if(global.mobility_mode == UHMMA_MODE2) 
			{
			    global.nbBACK = 3;
			    if (verboseU) 
					logtrace("\tUHMMA ; nbBACK %d", global.nbBACK);
			    COPY_ADDR6(*newcoa,global.HA->careof);
			    COPY_ADDR6(vcoa,global.HA->VCOA);
			}
			if(global.mobility_mode == UHMMA_MODE1)
			    update_cn_list();
		}
    }
}
/*
too munch retransmission of a uhmma registration was performed
we must asume that the local ma is down
so we have to swap to mobile ip.

- destroy the reemit timer
- free the uhmma registration packet
- check if the mobile is coming from its home network
- check the mobility mode

called by : resend_uhmma_reg
call : del_prefix, swap_mode, mobile_move
*/

void stop_UHMMA(coa)
struct in6_addr * coa;		// current careof
{
    global.nbBACK =0;
    /* destroy the reemission timer */
    DESTROY_TIMER(global.reemis_timer);
    /* free the reemit packet */
    free(global.reemit_reg_packet);
    /* the mobile is coming from its home network */
    if(IS_ANYADDR6(global.MA_old))
    {
		struct in6_addr svgcoa;
		struct prefix *vp;
		struct corr * cop;
	
		COPY_ADDR6(*coa,svgcoa);
		/* destroy the vcoa prefix */
		if(verboseU)
			logtrace("\tUHMMA ; destroy vcoa");
		FORALLVP(vp)
		{
		    del_prefix(vp,1);
		    TAILQ_REMOVE(&global.Vpref, vp, prf_list);
		}

        if(global.mobility_mode == UHMMA_MODE2)
		{
		    global.mobility_mode = MIP_MODE;
		    FORALLCOR(cop)
		    {
				/* simulation of a "move from home" */
				COPY_ADDR6(global.HA->home,cop->careof);
				CLR_ADDR6(cop->VCOA);
				mobile_move(cop,&svgcoa);
		    }
		}
		else /* UHMMA_MODE1		     */
		{
		    global.mobility_mode = MIP_MODE;
		    FORALLCOR(cop)
		    {
				/* simulation of a "move from a place to another" */
				COPY_ADDR6(global.VCOA,cop->careof);
				CLR_ADDR6(cop->VCOA);
				mobile_move(cop,&svgcoa);
	    	}
		}	
		clear_back_tug();
    }
    /* the mobile host is coming from an other domain*/
    else
    {
		char old_mode = global.mobility_mode;
		global.need_to_swap = MIP_MODE;
		swap_mode(coa);
		/* if the mobile runs mode 1, it has already send bu to all its
	   cn but some of them could contain vcoa
	   we have to resend new bu with coa
	   */
		if(old_mode == UHMMA_MODE1)
		{
		    struct corr * cop;
		    struct in6_addr svgcoa;
		    COPY_ADDR6(*coa,svgcoa);
		    FORALLCOR(cop)
	    	{
				CLR_ADDR6(cop->careof);
				mobile_move(cop,&svgcoa);
		    }
		}
    }
}
#endif
