#ifdef UHMMA

#include "ndpd.h"
#include "../ndpd-mobility/mobility.h"
#include "../ndpd-uhmma/uhmma.h"

void resend_smooth();
void stop_smooth(struct in6_addr *);

/*
the mobile host just moved. it was previously managed by using UHMMA 2.
now only MIP or Mode 1 is available. so we have to
send a smooth handoff packet to its previous ma.
the previous ma will send back the cn list in a back and will redirect
packets to the new position of the mobile host.

start a timer for restansmission.

called by : swap_mode
call : new_timeout, send_kernel
*/
void send_smooth(careof)
struct in6_addr * careof;	/* the new position of the mobile host */
{
	register u_char *cp;
	int byte_length = 48;
	int byte_8_length = (byte_length -8)/8;
	char bubuf[byte_length];

	/* update sequence# */
	global.seq_num ++;
	/*
	 * Fill a destination option header with the binding update
	 * and a home address options.
	 */
	cp = (u_char *)bubuf;

	SET8(IP6_NHDR_NONH, cp);
	SET8(byte_8_length, cp);

	SET8(OPT6_BINDING_UPD, cp);
	SET8(24, cp);
	SET8(SMOOTH_FLAG|0x20, cp);	/* C flag */
	SET8(0, cp);
	SET16(global.seq_num, cp);
	SET32(temp.forward, cp);/* PURSUIT_TIMEOUT < DEFAULT_MH_LIFETIME */
	SETADDR(*careof, cp); 

	SET8(OPT6_PAD_0, cp);
	SET8(OPT6_PAD_0, cp);

	SET8(OPT6_HOME_ADDR, cp);
	SET8(16, cp);
	SETADDR(global.HA->home, cp); /* home address */

	/* send */
	if(send_kernel("send smooth handoff",bubuf,byte_length,
			careof,&global.MA_old,0,0) == 0)
	{
	    free(bubuf);
	    return;
	}
	if (verboseU) 
	    logtrace("\tUHMMA ; sent smooth handoff to %s seq#%d",
			inet6pr(&global.MA_old),global.seq_num);
    /* retransmission */
    global.reg_ret =0;
    global.reemit_reg_packet= malloc(byte_length);
    if( global.reemit_reg_packet == NULL)
    {
	if(verboseU)
	    logtrace("\tUHMMA ; not enough memory for retransmission ");
	return;
    }
    memcpy(global.reemit_reg_packet, bubuf, byte_length);
    global.reemis_timer=new_timeout(temp.smooth, 0, (func_to)resend_smooth, NULL);
}

/*
resend the smooth handoff packet when the timer expires
this packet is stored in memory, 
the pointer on it is in the global structure.

restart the retransmission timer.

called by : run_timeout
call : set_timeout, send_kernel
*/
void resend_smooth()
{
	char * upd;

    /* upgrade seq# */
    upd = global.reemit_reg_packet + 6;
    global.seq_num++;
    SET16( global.seq_num, upd);

    /* check for retransmission */
    if(global.reg_ret ++ >= global.reg_ret_max)
    {
	if(verboseU)
	    logtrace("\tUHMMA ; too munch retransmissions");
	stop_smooth(&global.MA_old);
	return;
    }

	/* send */
	if(send_kernel("resend smooth",global.reemit_reg_packet,48,
		    &global.MA_old,&global.MA_address,0,0) == 0)
	/* note ; we use ma_old while swapping to back up the new careof
	    and MA_address to back up MA_old
	    not really good ... :-(
	*/
	    return;
	if (verboseU) 
		logtrace("\tUHMMA ; re sent smooth to %s seq#%d ret#%d", 
		    inet6pr(&global.MA_old),global.seq_num,
		    global.reg_ret);
    /* restart the retransmission timer */
    set_timeout(global.reemis_timer, temp.smooth, 0, (func_to)resend_smooth, NULL);
}
/*
we are running mode 2,
we performed a macro move but the new ma don't run mode 2 or no ma
so we try to swap by asking the cn list to the previous ma
but the previous ma does not respond
so we have to swap to mip or mode 1

called by : resend_smooth
call : 
*/
void stop_smooth(coa)
struct in6_addr *coa;	// current position
{
    struct corr *cop;
	
    if(verboseU)
	logtrace("\tUHMMA ; stop smooth handoff process");
    global.nbBACK =0;
    /* destroy the reemission timer */
    DESTROY_TIMER(global.reemis_timer);
    /* free the reemit packet */
    free(global.reemit_reg_packet);
    global.swap_in_progress = 0;
    global.nbBACK =0;
    // swap to MIP
    if(IS_ANYADDR6(global.MA_readed))
    {
	struct prefix *vp;

	global.mobility_mode = MIP_MODE;

	FORALLCOR(cop)
	{
	    COPY_ADDR6(cop->home,cop->careof);//simulation of a 'move from home'
	    cop->seq = 0x7000;	//try to bypass the seq#
	    CLR_ADDR6(cop->VCOA);
	    mobile_move(cop,coa);
	}
	global.seq_num =0;
	CLR_ADDR6(global.MA_address);
	CLR_ADDR6(global.MA_old);
	CLR_ADDR6(global.VCOA);
	/* depreciate the vcoa prefix */
	if(verboseU)
	    logtrace("\tUHMMA ; delete vcoa");
	FORALLVP(vp)
	{
	    del_prefix(vp,1);
	    TAILQ_REMOVE(&global.Vpref, vp, prf_list);
	}
    }
    // swap to MODE 1
    else
    {
	struct in6_addr svgcoa;
	
	COPY_ADDR6(*coa,svgcoa);
	global.mobility_mode = UHMMA_MODE1;
	
	COPY_ADDR6(global.MA_address,global.MA_old);
	COPY_ADDR6(global.MA_readed,global.MA_address);
	/* we register to the local MA, this MA will send a forward to the
	previous MA, this forward will probably be loosed */
	uhmma_move(&svgcoa);	
	update_cn_list();
	FORALLCOR(cop)
	{
	    COPY_ADDR6(cop->home,cop->careof); 
	    cop->seq = 0x7000;	//try to bypass the seq#
	    mobile_move(cop,&svgcoa);
	}
    }
}

#endif
