/*

UHMMA :

VCOA managment

*/
#ifdef UHMMA

#include "ndpd.h"
#include "../ndpd-mobility/mobility.h"
#include "uhmma.h"

char is_pseudo(struct interface * );



/*
add a vitual address on the  interface.
build the prefix with the VCOA addr and a prefix length of 128.
this address is built with the MA address, it is used to accept packets with
destination == VCOA.

we store the address of this structure in a global pointer, usefull to remove
this prefix when the mobile change domain.

note that we have to check if the itf is a physical device itf, 
and not a pseudo-device (like cti, sti).

called by : send_uhmma_major_reg, swap_mode
call : add_prefix, send_grat_adv, make_VCOA
*/
void add_VCOA(itf,vcoa)
struct interface * itf;		/* the interface */
struct in6_addr * vcoa;		/* the address to add */
{
    struct prefix *pp;

    /* check the name of the itf */
    if(is_pseudo(itf)==1)
	return;

    /* build the prefix */
    if (Calloc(pp, prefix) == NULL) 
    {
	if (verboseU)
		logerr("\tUHMMA ; out of memory, can't assign vcoas prefix");
	return;
    }
    if (verboseU)
	logtrace("\tUHMMA ; adding vcoa prefix %s on interface %s",
		 inet6pr(vcoa), itf->name);
    pp->ifp = itf;
    COPY_ADDR6(*vcoa, pp->pref);
    COPY_ADDR6(*vcoa, pp->addr);
    pp->plen = pp->alen = 128;
    mk_mask(&pp->pmask, 128);
    pp->invlife = INFLIFE;
    pp->deplife = DEFPREFLIFE;
    pp->pflags = pp->nflags = ND_OPT_PI_FLAG_AUTO;
    pp->mflags = MOB_ISVA;
    pp->ntosend = MAX_ADVERT_REXMIT;

    /* insert the prefix in the interface prefix list */
/*    TAILQ_INSERT_TAIL(&itf->prefixes, pp, prf_list);*/

    add_prefix(pp);
    /* bring this address to the first place */
/*    prompt(itf); */

    send_grat_adv(pp);

    /* save the prefix*/
//    itf->Vpref =pp;
    TAILQ_INSERT_TAIL(&global.Vpref, pp, prf_list);
}

/*
build a vcoa address by using the prefix of the MA address and the suffixe
of the llink address.

if the prefix length is 128 the local network is a CIP6 network, the vcoa
must be the MA address.

called by : send_uhmma_major_reg, swap_mode
call :
*/
struct in6_addr make_VCOA(coa)
struct in6_addr * coa;
{
    struct in6_addr ta;
    int i,j;
    /* build the vcoa */
    if (global.MA_plen == 128) 
    {
	return(global.MA_address);
    }
    CLR_ADDR6(ta);
    j = global.MA_plen / 8;
    for(i=0 ;i<j ;i++ )
        ta.s6_addr[i] = global.MA_address.s6_addr[i];
    for(i=j ;i<16 ;i++ )
        ta.s6_addr[i] = coa->s6_addr[i];
//        ta.s6_addr[i] = def_if->llocal.s6_addr[i];
    return ta;
}


/*
check if the interface is a pseudo device itf or a true physical itf
we use the name of the itf to determine its type.

called by : add_VCOA
call :
*/
char is_pseudo(itf)
struct interface * itf;	/* the interface to check */
{
    if(
	(itf->name[0] == 'c')&&
	(itf->name[0] == 't')&&
	(itf->name[0] == 'i')
      )    return 1;
    if(
	(itf->name[0] == 's')&&
	(itf->name[0] == 'i')&&
	(itf->name[0] == 't')
      )    return 1;
    return 0;
}
#endif
