#include "ndpd.h"
#include "../ndpd-mobility/mobility.h"
#ifdef UHMMA
#include "../ndpd-uhmma/uhmma.h"
#endif
#ifdef uMIP
#include "../ndpd-uhmma/uhmma.h"
#endif
#ifdef uMIPBS
#include "../ndpd-uhmma/uhmma.h"
#endif


#ifdef UHMMA
void mobile_uhmma_init()
{
	/* a mobile whitch runs UHMMA must :
	   - clear MA and old MA adresses, it is supposed to start 
	   at home
	   - init its first sequence number
	   - init the list of virtual prefix
	   - init the list of standard timers (can be modified by /etc/mobile6)
	 */
    CLR_ADDR6(global.MA_address);
    CLR_ADDR6(global.MA_old);
    CLR_ADDR6(global.VCOA);
    global.seq_num = 0;                        /* sequence number */
    TAILQ_INIT(&global.Vpref);
    global.need_to_swap = -1;
    global.swap_in_progress = 0;
    global.switch_in_progress = 0;
    global.site_len	= DEFAULT_SITE_LEN;
    global.reg_ret_max  = MAX_RETRANS;
    temp.home  		= BACK_TO_HOME_TIMEOUT;
    temp.lifetime  	= DEFAULT_MH_LIFETIME;
    temp.position  	= OLD_POSITION_TIMER;
    temp.refresh  	= DEFAULT_MH_REFRESH;
    temp.retransmission = REEMIT_REG_INTERVAL;
    temp.smooth  	= SMOOTH_TIMER;
	/* hack for ike */
	global.MA = malloc(sizeof(struct corr));
	if(global.MA == NULL)
	{
		if(verboseU)
			logtrace("not enought memory to store MA structure");
		exit (2);
	}
}
#endif

#ifdef uMIP
void ma_uhmma_init()
{
    /*
      If the router runs uMIP, it is a Base Station or a Mobility 
      Agent.
      There are two different mode for a MA, UHMMA1 and UHMMA2

      All MA must :
      - reserve memory for the MA annonce retransmission timer
      - start this timer for the very first time with a short period
      - initialize the lists of MH and tunnels
      - initialize the Virtual Prefix structure
      - initialize the list of timeout with standard values, this values 
        can be modified by read_mobile (/etc/gateway).

      A MA in UHMMA2 must :
      - initialize the list of Waiting BAck (for smooth handoff)

      A BS must :
      - initialize the MA_address

     the virtual prefix, the name and the length are readed in read_gateway.
	we just stored them in the VirtualNetwork structure.
	now we need to affect the pointers to the interface structure, and to
	the prefix structure.
    
     */
    if((MA_op ==1)||(MA_op ==2))
    {
        MA_annonce_timer= new_timeout(5, 0, (func_to) send_MA_annonce, 0);
      
	if (MA_op == 2)
	    TAILQ_INIT(&waitingBack);

	TAILQ_INIT(&listMH);
	TAILQ_INIT(&listBS);
	TAILQ_INIT(&listMA);
	charge =0;
	TAILQ_INIT(&tunnels);
	init_tunnels();
	temp.annonce		= MA_ANNONCE_TIMER;
	temp.home  		= BACK_TO_HOME_TIMEOUT;
	temp.lifetime  		= DEFAULT_MH_LIFETIME;
	temp.refresh  		= DEFAULT_MH_REFRESH;
	temp.smooth  		= SMOOTH_TIMER;
	temp.forward		= PURSUIT_TIMEOUT;
	max_retrans 		= MAX_RETRANS;
	site_len                = DEFAULT_SITE_LEN;
	strcpy(VirtualNetwork.name,"");
	VirtualNetwork.plen	= 0;
	CLR_ADDR6(VirtualNetwork.pref);
    }
}
#endif

#ifdef uMIPBS
void bs_uhmma_init()
{
    if(BS_op == 1)
    {
	CLR_ADDR6(MA_info.MA_address);
    }
}
#endif


#ifdef UHMMA
void mobile_uhmma_clean()
{
    /* if the MH tuns UHMMA and if the mobile is not at home
	send a de registration packet to the local MA (ie a registration
	packet with a life field equal to 0
     */
    if(
	(global.mobility_mode == UHMMA_MODE1)||
	(global.mobility_mode == UHMMA_MODE2)
      )
	if(FIRST(def_if)->mflags != MOB_ISHA)
	{
	    if(verboseU)
		logtrace("\tUHMMA ; send deregistration pkt to local MA");
	    build_send_reg(&global.HA->careof,0,0,1);
	}
    /* remove the virtual prefix  */
    {
		struct prefix * vp;
	   	FORALLVP(vp)
		{
		    del_prefix(vp,1);
		    TAILQ_REMOVE(&global.Vpref,vp,prf_list);
		    free(vp);
		}
    }
	/* hack for ike */
	del_homeaddr(global.MA);
	free(global.MA);
}

#endif

#ifdef uMIP
void ma_uhmma_clean()
{
    /* destroy all binding in the kernel for registered mobile
       close all tunnels to mobile 
       remove n free all mobile structures
     */
    struct mobile * mop;
    if(MA_op > 0)
	FORALLMAMH(mop)
	    kill_mobile(mop);
    if(MA_op ==2)
	close_bpf();
    /* say to the BSs that we stop  */
    send_MA_annonce(1);
    DESTROY_TIMER(MA_annonce_timer);
}
#endif

#ifdef uMIPBS
void bs_uhmma_clean()
{
}
#endif
